from enum import Enum
import math
import string


class TorquePerLengthUnits(Enum):
        """
            TorquePerLengthUnits enumeration
        """
        
        NewtonMillimeterPerMeter = 'newton_millimeter_per_meter'
        """
            
        """
        
        NewtonCentimeterPerMeter = 'newton_centimeter_per_meter'
        """
            
        """
        
        NewtonMeterPerMeter = 'newton_meter_per_meter'
        """
            
        """
        
        PoundForceInchPerFoot = 'pound_force_inch_per_foot'
        """
            
        """
        
        PoundForceFootPerFoot = 'pound_force_foot_per_foot'
        """
            
        """
        
        KilogramForceMillimeterPerMeter = 'kilogram_force_millimeter_per_meter'
        """
            
        """
        
        KilogramForceCentimeterPerMeter = 'kilogram_force_centimeter_per_meter'
        """
            
        """
        
        KilogramForceMeterPerMeter = 'kilogram_force_meter_per_meter'
        """
            
        """
        
        TonneForceMillimeterPerMeter = 'tonne_force_millimeter_per_meter'
        """
            
        """
        
        TonneForceCentimeterPerMeter = 'tonne_force_centimeter_per_meter'
        """
            
        """
        
        TonneForceMeterPerMeter = 'tonne_force_meter_per_meter'
        """
            
        """
        
        KilonewtonMillimeterPerMeter = 'kilonewton_millimeter_per_meter'
        """
            
        """
        
        MeganewtonMillimeterPerMeter = 'meganewton_millimeter_per_meter'
        """
            
        """
        
        KilonewtonCentimeterPerMeter = 'kilonewton_centimeter_per_meter'
        """
            
        """
        
        MeganewtonCentimeterPerMeter = 'meganewton_centimeter_per_meter'
        """
            
        """
        
        KilonewtonMeterPerMeter = 'kilonewton_meter_per_meter'
        """
            
        """
        
        MeganewtonMeterPerMeter = 'meganewton_meter_per_meter'
        """
            
        """
        
        KilopoundForceInchPerFoot = 'kilopound_force_inch_per_foot'
        """
            
        """
        
        MegapoundForceInchPerFoot = 'megapound_force_inch_per_foot'
        """
            
        """
        
        KilopoundForceFootPerFoot = 'kilopound_force_foot_per_foot'
        """
            
        """
        
        MegapoundForceFootPerFoot = 'megapound_force_foot_per_foot'
        """
            
        """
        

class TorquePerLength:
    """
    The magnitude of torque per unit length.

    Args:
        value (float): The value.
        from_unit (TorquePerLengthUnits): The TorquePerLength unit to create from, The default unit is NewtonMeterPerMeter
    """
    def __init__(self, value: float, from_unit: TorquePerLengthUnits = TorquePerLengthUnits.NewtonMeterPerMeter):
        if math.isnan(value):
            raise ValueError('Invalid unit: value is NaN')
        self.__value = self.__convert_to_base(value, from_unit)
        
        self.__newton_millimeters_per_meter = None
        
        self.__newton_centimeters_per_meter = None
        
        self.__newton_meters_per_meter = None
        
        self.__pound_force_inches_per_foot = None
        
        self.__pound_force_feet_per_foot = None
        
        self.__kilogram_force_millimeters_per_meter = None
        
        self.__kilogram_force_centimeters_per_meter = None
        
        self.__kilogram_force_meters_per_meter = None
        
        self.__tonne_force_millimeters_per_meter = None
        
        self.__tonne_force_centimeters_per_meter = None
        
        self.__tonne_force_meters_per_meter = None
        
        self.__kilonewton_millimeters_per_meter = None
        
        self.__meganewton_millimeters_per_meter = None
        
        self.__kilonewton_centimeters_per_meter = None
        
        self.__meganewton_centimeters_per_meter = None
        
        self.__kilonewton_meters_per_meter = None
        
        self.__meganewton_meters_per_meter = None
        
        self.__kilopound_force_inches_per_foot = None
        
        self.__megapound_force_inches_per_foot = None
        
        self.__kilopound_force_feet_per_foot = None
        
        self.__megapound_force_feet_per_foot = None
        

    def __convert_from_base(self, from_unit: TorquePerLengthUnits) -> float:
        value = self.__value
        
        if from_unit == TorquePerLengthUnits.NewtonMillimeterPerMeter:
            return (value * 1000)
        
        if from_unit == TorquePerLengthUnits.NewtonCentimeterPerMeter:
            return (value * 100)
        
        if from_unit == TorquePerLengthUnits.NewtonMeterPerMeter:
            return (value)
        
        if from_unit == TorquePerLengthUnits.PoundForceInchPerFoot:
            return (value / 0.370685147638)
        
        if from_unit == TorquePerLengthUnits.PoundForceFootPerFoot:
            return (value / 4.44822161526)
        
        if from_unit == TorquePerLengthUnits.KilogramForceMillimeterPerMeter:
            return (value * 101.971619222242)
        
        if from_unit == TorquePerLengthUnits.KilogramForceCentimeterPerMeter:
            return (value * 10.1971619222242)
        
        if from_unit == TorquePerLengthUnits.KilogramForceMeterPerMeter:
            return (value * 0.101971619222242)
        
        if from_unit == TorquePerLengthUnits.TonneForceMillimeterPerMeter:
            return (value * 0.101971619222242)
        
        if from_unit == TorquePerLengthUnits.TonneForceCentimeterPerMeter:
            return (value * 0.0101971619222242)
        
        if from_unit == TorquePerLengthUnits.TonneForceMeterPerMeter:
            return (value * 0.000101971619222242)
        
        if from_unit == TorquePerLengthUnits.KilonewtonMillimeterPerMeter:
            return ((value * 1000) / 1000.0)
        
        if from_unit == TorquePerLengthUnits.MeganewtonMillimeterPerMeter:
            return ((value * 1000) / 1000000.0)
        
        if from_unit == TorquePerLengthUnits.KilonewtonCentimeterPerMeter:
            return ((value * 100) / 1000.0)
        
        if from_unit == TorquePerLengthUnits.MeganewtonCentimeterPerMeter:
            return ((value * 100) / 1000000.0)
        
        if from_unit == TorquePerLengthUnits.KilonewtonMeterPerMeter:
            return ((value) / 1000.0)
        
        if from_unit == TorquePerLengthUnits.MeganewtonMeterPerMeter:
            return ((value) / 1000000.0)
        
        if from_unit == TorquePerLengthUnits.KilopoundForceInchPerFoot:
            return ((value / 0.370685147638) / 1000.0)
        
        if from_unit == TorquePerLengthUnits.MegapoundForceInchPerFoot:
            return ((value / 0.370685147638) / 1000000.0)
        
        if from_unit == TorquePerLengthUnits.KilopoundForceFootPerFoot:
            return ((value / 4.44822161526) / 1000.0)
        
        if from_unit == TorquePerLengthUnits.MegapoundForceFootPerFoot:
            return ((value / 4.44822161526) / 1000000.0)
        
        return None


    def __convert_to_base(self, value: float, to_unit: TorquePerLengthUnits) -> float:
        
        if to_unit == TorquePerLengthUnits.NewtonMillimeterPerMeter:
            return (value * 0.001)
        
        if to_unit == TorquePerLengthUnits.NewtonCentimeterPerMeter:
            return (value * 0.01)
        
        if to_unit == TorquePerLengthUnits.NewtonMeterPerMeter:
            return (value)
        
        if to_unit == TorquePerLengthUnits.PoundForceInchPerFoot:
            return (value * 0.370685147638)
        
        if to_unit == TorquePerLengthUnits.PoundForceFootPerFoot:
            return (value * 4.44822161526)
        
        if to_unit == TorquePerLengthUnits.KilogramForceMillimeterPerMeter:
            return (value * 0.00980665019960652)
        
        if to_unit == TorquePerLengthUnits.KilogramForceCentimeterPerMeter:
            return (value * 0.0980665019960652)
        
        if to_unit == TorquePerLengthUnits.KilogramForceMeterPerMeter:
            return (value * 9.80665019960652)
        
        if to_unit == TorquePerLengthUnits.TonneForceMillimeterPerMeter:
            return (value * 9.80665019960652)
        
        if to_unit == TorquePerLengthUnits.TonneForceCentimeterPerMeter:
            return (value * 98.0665019960652)
        
        if to_unit == TorquePerLengthUnits.TonneForceMeterPerMeter:
            return (value * 9806.65019960653)
        
        if to_unit == TorquePerLengthUnits.KilonewtonMillimeterPerMeter:
            return ((value * 0.001) * 1000.0)
        
        if to_unit == TorquePerLengthUnits.MeganewtonMillimeterPerMeter:
            return ((value * 0.001) * 1000000.0)
        
        if to_unit == TorquePerLengthUnits.KilonewtonCentimeterPerMeter:
            return ((value * 0.01) * 1000.0)
        
        if to_unit == TorquePerLengthUnits.MeganewtonCentimeterPerMeter:
            return ((value * 0.01) * 1000000.0)
        
        if to_unit == TorquePerLengthUnits.KilonewtonMeterPerMeter:
            return ((value) * 1000.0)
        
        if to_unit == TorquePerLengthUnits.MeganewtonMeterPerMeter:
            return ((value) * 1000000.0)
        
        if to_unit == TorquePerLengthUnits.KilopoundForceInchPerFoot:
            return ((value * 0.370685147638) * 1000.0)
        
        if to_unit == TorquePerLengthUnits.MegapoundForceInchPerFoot:
            return ((value * 0.370685147638) * 1000000.0)
        
        if to_unit == TorquePerLengthUnits.KilopoundForceFootPerFoot:
            return ((value * 4.44822161526) * 1000.0)
        
        if to_unit == TorquePerLengthUnits.MegapoundForceFootPerFoot:
            return ((value * 4.44822161526) * 1000000.0)
        
        return None


    @property
    def base_value(self) -> float:
        return self.__value

    
    @staticmethod
    def from_newton_millimeters_per_meter(newton_millimeters_per_meter: float):
        """
        Create a new instance of TorquePerLength from a value in newton_millimeters_per_meter.

        

        :param meters: The TorquePerLength value in newton_millimeters_per_meter.
        :type newton_millimeters_per_meter: float
        :return: A new instance of TorquePerLength.
        :rtype: TorquePerLength
        """
        return TorquePerLength(newton_millimeters_per_meter, TorquePerLengthUnits.NewtonMillimeterPerMeter)

    
    @staticmethod
    def from_newton_centimeters_per_meter(newton_centimeters_per_meter: float):
        """
        Create a new instance of TorquePerLength from a value in newton_centimeters_per_meter.

        

        :param meters: The TorquePerLength value in newton_centimeters_per_meter.
        :type newton_centimeters_per_meter: float
        :return: A new instance of TorquePerLength.
        :rtype: TorquePerLength
        """
        return TorquePerLength(newton_centimeters_per_meter, TorquePerLengthUnits.NewtonCentimeterPerMeter)

    
    @staticmethod
    def from_newton_meters_per_meter(newton_meters_per_meter: float):
        """
        Create a new instance of TorquePerLength from a value in newton_meters_per_meter.

        

        :param meters: The TorquePerLength value in newton_meters_per_meter.
        :type newton_meters_per_meter: float
        :return: A new instance of TorquePerLength.
        :rtype: TorquePerLength
        """
        return TorquePerLength(newton_meters_per_meter, TorquePerLengthUnits.NewtonMeterPerMeter)

    
    @staticmethod
    def from_pound_force_inches_per_foot(pound_force_inches_per_foot: float):
        """
        Create a new instance of TorquePerLength from a value in pound_force_inches_per_foot.

        

        :param meters: The TorquePerLength value in pound_force_inches_per_foot.
        :type pound_force_inches_per_foot: float
        :return: A new instance of TorquePerLength.
        :rtype: TorquePerLength
        """
        return TorquePerLength(pound_force_inches_per_foot, TorquePerLengthUnits.PoundForceInchPerFoot)

    
    @staticmethod
    def from_pound_force_feet_per_foot(pound_force_feet_per_foot: float):
        """
        Create a new instance of TorquePerLength from a value in pound_force_feet_per_foot.

        

        :param meters: The TorquePerLength value in pound_force_feet_per_foot.
        :type pound_force_feet_per_foot: float
        :return: A new instance of TorquePerLength.
        :rtype: TorquePerLength
        """
        return TorquePerLength(pound_force_feet_per_foot, TorquePerLengthUnits.PoundForceFootPerFoot)

    
    @staticmethod
    def from_kilogram_force_millimeters_per_meter(kilogram_force_millimeters_per_meter: float):
        """
        Create a new instance of TorquePerLength from a value in kilogram_force_millimeters_per_meter.

        

        :param meters: The TorquePerLength value in kilogram_force_millimeters_per_meter.
        :type kilogram_force_millimeters_per_meter: float
        :return: A new instance of TorquePerLength.
        :rtype: TorquePerLength
        """
        return TorquePerLength(kilogram_force_millimeters_per_meter, TorquePerLengthUnits.KilogramForceMillimeterPerMeter)

    
    @staticmethod
    def from_kilogram_force_centimeters_per_meter(kilogram_force_centimeters_per_meter: float):
        """
        Create a new instance of TorquePerLength from a value in kilogram_force_centimeters_per_meter.

        

        :param meters: The TorquePerLength value in kilogram_force_centimeters_per_meter.
        :type kilogram_force_centimeters_per_meter: float
        :return: A new instance of TorquePerLength.
        :rtype: TorquePerLength
        """
        return TorquePerLength(kilogram_force_centimeters_per_meter, TorquePerLengthUnits.KilogramForceCentimeterPerMeter)

    
    @staticmethod
    def from_kilogram_force_meters_per_meter(kilogram_force_meters_per_meter: float):
        """
        Create a new instance of TorquePerLength from a value in kilogram_force_meters_per_meter.

        

        :param meters: The TorquePerLength value in kilogram_force_meters_per_meter.
        :type kilogram_force_meters_per_meter: float
        :return: A new instance of TorquePerLength.
        :rtype: TorquePerLength
        """
        return TorquePerLength(kilogram_force_meters_per_meter, TorquePerLengthUnits.KilogramForceMeterPerMeter)

    
    @staticmethod
    def from_tonne_force_millimeters_per_meter(tonne_force_millimeters_per_meter: float):
        """
        Create a new instance of TorquePerLength from a value in tonne_force_millimeters_per_meter.

        

        :param meters: The TorquePerLength value in tonne_force_millimeters_per_meter.
        :type tonne_force_millimeters_per_meter: float
        :return: A new instance of TorquePerLength.
        :rtype: TorquePerLength
        """
        return TorquePerLength(tonne_force_millimeters_per_meter, TorquePerLengthUnits.TonneForceMillimeterPerMeter)

    
    @staticmethod
    def from_tonne_force_centimeters_per_meter(tonne_force_centimeters_per_meter: float):
        """
        Create a new instance of TorquePerLength from a value in tonne_force_centimeters_per_meter.

        

        :param meters: The TorquePerLength value in tonne_force_centimeters_per_meter.
        :type tonne_force_centimeters_per_meter: float
        :return: A new instance of TorquePerLength.
        :rtype: TorquePerLength
        """
        return TorquePerLength(tonne_force_centimeters_per_meter, TorquePerLengthUnits.TonneForceCentimeterPerMeter)

    
    @staticmethod
    def from_tonne_force_meters_per_meter(tonne_force_meters_per_meter: float):
        """
        Create a new instance of TorquePerLength from a value in tonne_force_meters_per_meter.

        

        :param meters: The TorquePerLength value in tonne_force_meters_per_meter.
        :type tonne_force_meters_per_meter: float
        :return: A new instance of TorquePerLength.
        :rtype: TorquePerLength
        """
        return TorquePerLength(tonne_force_meters_per_meter, TorquePerLengthUnits.TonneForceMeterPerMeter)

    
    @staticmethod
    def from_kilonewton_millimeters_per_meter(kilonewton_millimeters_per_meter: float):
        """
        Create a new instance of TorquePerLength from a value in kilonewton_millimeters_per_meter.

        

        :param meters: The TorquePerLength value in kilonewton_millimeters_per_meter.
        :type kilonewton_millimeters_per_meter: float
        :return: A new instance of TorquePerLength.
        :rtype: TorquePerLength
        """
        return TorquePerLength(kilonewton_millimeters_per_meter, TorquePerLengthUnits.KilonewtonMillimeterPerMeter)

    
    @staticmethod
    def from_meganewton_millimeters_per_meter(meganewton_millimeters_per_meter: float):
        """
        Create a new instance of TorquePerLength from a value in meganewton_millimeters_per_meter.

        

        :param meters: The TorquePerLength value in meganewton_millimeters_per_meter.
        :type meganewton_millimeters_per_meter: float
        :return: A new instance of TorquePerLength.
        :rtype: TorquePerLength
        """
        return TorquePerLength(meganewton_millimeters_per_meter, TorquePerLengthUnits.MeganewtonMillimeterPerMeter)

    
    @staticmethod
    def from_kilonewton_centimeters_per_meter(kilonewton_centimeters_per_meter: float):
        """
        Create a new instance of TorquePerLength from a value in kilonewton_centimeters_per_meter.

        

        :param meters: The TorquePerLength value in kilonewton_centimeters_per_meter.
        :type kilonewton_centimeters_per_meter: float
        :return: A new instance of TorquePerLength.
        :rtype: TorquePerLength
        """
        return TorquePerLength(kilonewton_centimeters_per_meter, TorquePerLengthUnits.KilonewtonCentimeterPerMeter)

    
    @staticmethod
    def from_meganewton_centimeters_per_meter(meganewton_centimeters_per_meter: float):
        """
        Create a new instance of TorquePerLength from a value in meganewton_centimeters_per_meter.

        

        :param meters: The TorquePerLength value in meganewton_centimeters_per_meter.
        :type meganewton_centimeters_per_meter: float
        :return: A new instance of TorquePerLength.
        :rtype: TorquePerLength
        """
        return TorquePerLength(meganewton_centimeters_per_meter, TorquePerLengthUnits.MeganewtonCentimeterPerMeter)

    
    @staticmethod
    def from_kilonewton_meters_per_meter(kilonewton_meters_per_meter: float):
        """
        Create a new instance of TorquePerLength from a value in kilonewton_meters_per_meter.

        

        :param meters: The TorquePerLength value in kilonewton_meters_per_meter.
        :type kilonewton_meters_per_meter: float
        :return: A new instance of TorquePerLength.
        :rtype: TorquePerLength
        """
        return TorquePerLength(kilonewton_meters_per_meter, TorquePerLengthUnits.KilonewtonMeterPerMeter)

    
    @staticmethod
    def from_meganewton_meters_per_meter(meganewton_meters_per_meter: float):
        """
        Create a new instance of TorquePerLength from a value in meganewton_meters_per_meter.

        

        :param meters: The TorquePerLength value in meganewton_meters_per_meter.
        :type meganewton_meters_per_meter: float
        :return: A new instance of TorquePerLength.
        :rtype: TorquePerLength
        """
        return TorquePerLength(meganewton_meters_per_meter, TorquePerLengthUnits.MeganewtonMeterPerMeter)

    
    @staticmethod
    def from_kilopound_force_inches_per_foot(kilopound_force_inches_per_foot: float):
        """
        Create a new instance of TorquePerLength from a value in kilopound_force_inches_per_foot.

        

        :param meters: The TorquePerLength value in kilopound_force_inches_per_foot.
        :type kilopound_force_inches_per_foot: float
        :return: A new instance of TorquePerLength.
        :rtype: TorquePerLength
        """
        return TorquePerLength(kilopound_force_inches_per_foot, TorquePerLengthUnits.KilopoundForceInchPerFoot)

    
    @staticmethod
    def from_megapound_force_inches_per_foot(megapound_force_inches_per_foot: float):
        """
        Create a new instance of TorquePerLength from a value in megapound_force_inches_per_foot.

        

        :param meters: The TorquePerLength value in megapound_force_inches_per_foot.
        :type megapound_force_inches_per_foot: float
        :return: A new instance of TorquePerLength.
        :rtype: TorquePerLength
        """
        return TorquePerLength(megapound_force_inches_per_foot, TorquePerLengthUnits.MegapoundForceInchPerFoot)

    
    @staticmethod
    def from_kilopound_force_feet_per_foot(kilopound_force_feet_per_foot: float):
        """
        Create a new instance of TorquePerLength from a value in kilopound_force_feet_per_foot.

        

        :param meters: The TorquePerLength value in kilopound_force_feet_per_foot.
        :type kilopound_force_feet_per_foot: float
        :return: A new instance of TorquePerLength.
        :rtype: TorquePerLength
        """
        return TorquePerLength(kilopound_force_feet_per_foot, TorquePerLengthUnits.KilopoundForceFootPerFoot)

    
    @staticmethod
    def from_megapound_force_feet_per_foot(megapound_force_feet_per_foot: float):
        """
        Create a new instance of TorquePerLength from a value in megapound_force_feet_per_foot.

        

        :param meters: The TorquePerLength value in megapound_force_feet_per_foot.
        :type megapound_force_feet_per_foot: float
        :return: A new instance of TorquePerLength.
        :rtype: TorquePerLength
        """
        return TorquePerLength(megapound_force_feet_per_foot, TorquePerLengthUnits.MegapoundForceFootPerFoot)

    
    @property
    def newton_millimeters_per_meter(self) -> float:
        """
        
        """
        if self.__newton_millimeters_per_meter != None:
            return self.__newton_millimeters_per_meter
        self.__newton_millimeters_per_meter = self.__convert_from_base(TorquePerLengthUnits.NewtonMillimeterPerMeter)
        return self.__newton_millimeters_per_meter

    
    @property
    def newton_centimeters_per_meter(self) -> float:
        """
        
        """
        if self.__newton_centimeters_per_meter != None:
            return self.__newton_centimeters_per_meter
        self.__newton_centimeters_per_meter = self.__convert_from_base(TorquePerLengthUnits.NewtonCentimeterPerMeter)
        return self.__newton_centimeters_per_meter

    
    @property
    def newton_meters_per_meter(self) -> float:
        """
        
        """
        if self.__newton_meters_per_meter != None:
            return self.__newton_meters_per_meter
        self.__newton_meters_per_meter = self.__convert_from_base(TorquePerLengthUnits.NewtonMeterPerMeter)
        return self.__newton_meters_per_meter

    
    @property
    def pound_force_inches_per_foot(self) -> float:
        """
        
        """
        if self.__pound_force_inches_per_foot != None:
            return self.__pound_force_inches_per_foot
        self.__pound_force_inches_per_foot = self.__convert_from_base(TorquePerLengthUnits.PoundForceInchPerFoot)
        return self.__pound_force_inches_per_foot

    
    @property
    def pound_force_feet_per_foot(self) -> float:
        """
        
        """
        if self.__pound_force_feet_per_foot != None:
            return self.__pound_force_feet_per_foot
        self.__pound_force_feet_per_foot = self.__convert_from_base(TorquePerLengthUnits.PoundForceFootPerFoot)
        return self.__pound_force_feet_per_foot

    
    @property
    def kilogram_force_millimeters_per_meter(self) -> float:
        """
        
        """
        if self.__kilogram_force_millimeters_per_meter != None:
            return self.__kilogram_force_millimeters_per_meter
        self.__kilogram_force_millimeters_per_meter = self.__convert_from_base(TorquePerLengthUnits.KilogramForceMillimeterPerMeter)
        return self.__kilogram_force_millimeters_per_meter

    
    @property
    def kilogram_force_centimeters_per_meter(self) -> float:
        """
        
        """
        if self.__kilogram_force_centimeters_per_meter != None:
            return self.__kilogram_force_centimeters_per_meter
        self.__kilogram_force_centimeters_per_meter = self.__convert_from_base(TorquePerLengthUnits.KilogramForceCentimeterPerMeter)
        return self.__kilogram_force_centimeters_per_meter

    
    @property
    def kilogram_force_meters_per_meter(self) -> float:
        """
        
        """
        if self.__kilogram_force_meters_per_meter != None:
            return self.__kilogram_force_meters_per_meter
        self.__kilogram_force_meters_per_meter = self.__convert_from_base(TorquePerLengthUnits.KilogramForceMeterPerMeter)
        return self.__kilogram_force_meters_per_meter

    
    @property
    def tonne_force_millimeters_per_meter(self) -> float:
        """
        
        """
        if self.__tonne_force_millimeters_per_meter != None:
            return self.__tonne_force_millimeters_per_meter
        self.__tonne_force_millimeters_per_meter = self.__convert_from_base(TorquePerLengthUnits.TonneForceMillimeterPerMeter)
        return self.__tonne_force_millimeters_per_meter

    
    @property
    def tonne_force_centimeters_per_meter(self) -> float:
        """
        
        """
        if self.__tonne_force_centimeters_per_meter != None:
            return self.__tonne_force_centimeters_per_meter
        self.__tonne_force_centimeters_per_meter = self.__convert_from_base(TorquePerLengthUnits.TonneForceCentimeterPerMeter)
        return self.__tonne_force_centimeters_per_meter

    
    @property
    def tonne_force_meters_per_meter(self) -> float:
        """
        
        """
        if self.__tonne_force_meters_per_meter != None:
            return self.__tonne_force_meters_per_meter
        self.__tonne_force_meters_per_meter = self.__convert_from_base(TorquePerLengthUnits.TonneForceMeterPerMeter)
        return self.__tonne_force_meters_per_meter

    
    @property
    def kilonewton_millimeters_per_meter(self) -> float:
        """
        
        """
        if self.__kilonewton_millimeters_per_meter != None:
            return self.__kilonewton_millimeters_per_meter
        self.__kilonewton_millimeters_per_meter = self.__convert_from_base(TorquePerLengthUnits.KilonewtonMillimeterPerMeter)
        return self.__kilonewton_millimeters_per_meter

    
    @property
    def meganewton_millimeters_per_meter(self) -> float:
        """
        
        """
        if self.__meganewton_millimeters_per_meter != None:
            return self.__meganewton_millimeters_per_meter
        self.__meganewton_millimeters_per_meter = self.__convert_from_base(TorquePerLengthUnits.MeganewtonMillimeterPerMeter)
        return self.__meganewton_millimeters_per_meter

    
    @property
    def kilonewton_centimeters_per_meter(self) -> float:
        """
        
        """
        if self.__kilonewton_centimeters_per_meter != None:
            return self.__kilonewton_centimeters_per_meter
        self.__kilonewton_centimeters_per_meter = self.__convert_from_base(TorquePerLengthUnits.KilonewtonCentimeterPerMeter)
        return self.__kilonewton_centimeters_per_meter

    
    @property
    def meganewton_centimeters_per_meter(self) -> float:
        """
        
        """
        if self.__meganewton_centimeters_per_meter != None:
            return self.__meganewton_centimeters_per_meter
        self.__meganewton_centimeters_per_meter = self.__convert_from_base(TorquePerLengthUnits.MeganewtonCentimeterPerMeter)
        return self.__meganewton_centimeters_per_meter

    
    @property
    def kilonewton_meters_per_meter(self) -> float:
        """
        
        """
        if self.__kilonewton_meters_per_meter != None:
            return self.__kilonewton_meters_per_meter
        self.__kilonewton_meters_per_meter = self.__convert_from_base(TorquePerLengthUnits.KilonewtonMeterPerMeter)
        return self.__kilonewton_meters_per_meter

    
    @property
    def meganewton_meters_per_meter(self) -> float:
        """
        
        """
        if self.__meganewton_meters_per_meter != None:
            return self.__meganewton_meters_per_meter
        self.__meganewton_meters_per_meter = self.__convert_from_base(TorquePerLengthUnits.MeganewtonMeterPerMeter)
        return self.__meganewton_meters_per_meter

    
    @property
    def kilopound_force_inches_per_foot(self) -> float:
        """
        
        """
        if self.__kilopound_force_inches_per_foot != None:
            return self.__kilopound_force_inches_per_foot
        self.__kilopound_force_inches_per_foot = self.__convert_from_base(TorquePerLengthUnits.KilopoundForceInchPerFoot)
        return self.__kilopound_force_inches_per_foot

    
    @property
    def megapound_force_inches_per_foot(self) -> float:
        """
        
        """
        if self.__megapound_force_inches_per_foot != None:
            return self.__megapound_force_inches_per_foot
        self.__megapound_force_inches_per_foot = self.__convert_from_base(TorquePerLengthUnits.MegapoundForceInchPerFoot)
        return self.__megapound_force_inches_per_foot

    
    @property
    def kilopound_force_feet_per_foot(self) -> float:
        """
        
        """
        if self.__kilopound_force_feet_per_foot != None:
            return self.__kilopound_force_feet_per_foot
        self.__kilopound_force_feet_per_foot = self.__convert_from_base(TorquePerLengthUnits.KilopoundForceFootPerFoot)
        return self.__kilopound_force_feet_per_foot

    
    @property
    def megapound_force_feet_per_foot(self) -> float:
        """
        
        """
        if self.__megapound_force_feet_per_foot != None:
            return self.__megapound_force_feet_per_foot
        self.__megapound_force_feet_per_foot = self.__convert_from_base(TorquePerLengthUnits.MegapoundForceFootPerFoot)
        return self.__megapound_force_feet_per_foot

    
    def to_string(self, unit: TorquePerLengthUnits = TorquePerLengthUnits.NewtonMeterPerMeter) -> string:
        """
        Format the TorquePerLength to string.
        Note! the default format for TorquePerLength is NewtonMeterPerMeter.
        To specify the unit format set the 'unit' parameter.
        """
        
        if unit == TorquePerLengthUnits.NewtonMillimeterPerMeter:
            return f"""{self.newton_millimeters_per_meter} N·mm/m"""
        
        if unit == TorquePerLengthUnits.NewtonCentimeterPerMeter:
            return f"""{self.newton_centimeters_per_meter} N·cm/m"""
        
        if unit == TorquePerLengthUnits.NewtonMeterPerMeter:
            return f"""{self.newton_meters_per_meter} N·m/m"""
        
        if unit == TorquePerLengthUnits.PoundForceInchPerFoot:
            return f"""{self.pound_force_inches_per_foot} lbf·in/ft"""
        
        if unit == TorquePerLengthUnits.PoundForceFootPerFoot:
            return f"""{self.pound_force_feet_per_foot} lbf·ft/ft"""
        
        if unit == TorquePerLengthUnits.KilogramForceMillimeterPerMeter:
            return f"""{self.kilogram_force_millimeters_per_meter} kgf·mm/m"""
        
        if unit == TorquePerLengthUnits.KilogramForceCentimeterPerMeter:
            return f"""{self.kilogram_force_centimeters_per_meter} kgf·cm/m"""
        
        if unit == TorquePerLengthUnits.KilogramForceMeterPerMeter:
            return f"""{self.kilogram_force_meters_per_meter} kgf·m/m"""
        
        if unit == TorquePerLengthUnits.TonneForceMillimeterPerMeter:
            return f"""{self.tonne_force_millimeters_per_meter} tf·mm/m"""
        
        if unit == TorquePerLengthUnits.TonneForceCentimeterPerMeter:
            return f"""{self.tonne_force_centimeters_per_meter} tf·cm/m"""
        
        if unit == TorquePerLengthUnits.TonneForceMeterPerMeter:
            return f"""{self.tonne_force_meters_per_meter} tf·m/m"""
        
        if unit == TorquePerLengthUnits.KilonewtonMillimeterPerMeter:
            return f"""{self.kilonewton_millimeters_per_meter} """
        
        if unit == TorquePerLengthUnits.MeganewtonMillimeterPerMeter:
            return f"""{self.meganewton_millimeters_per_meter} """
        
        if unit == TorquePerLengthUnits.KilonewtonCentimeterPerMeter:
            return f"""{self.kilonewton_centimeters_per_meter} """
        
        if unit == TorquePerLengthUnits.MeganewtonCentimeterPerMeter:
            return f"""{self.meganewton_centimeters_per_meter} """
        
        if unit == TorquePerLengthUnits.KilonewtonMeterPerMeter:
            return f"""{self.kilonewton_meters_per_meter} """
        
        if unit == TorquePerLengthUnits.MeganewtonMeterPerMeter:
            return f"""{self.meganewton_meters_per_meter} """
        
        if unit == TorquePerLengthUnits.KilopoundForceInchPerFoot:
            return f"""{self.kilopound_force_inches_per_foot} """
        
        if unit == TorquePerLengthUnits.MegapoundForceInchPerFoot:
            return f"""{self.megapound_force_inches_per_foot} """
        
        if unit == TorquePerLengthUnits.KilopoundForceFootPerFoot:
            return f"""{self.kilopound_force_feet_per_foot} """
        
        if unit == TorquePerLengthUnits.MegapoundForceFootPerFoot:
            return f"""{self.megapound_force_feet_per_foot} """
        
        return f'{self.__value}'


    def get_unit_abbreviation(self, unit_abbreviation: TorquePerLengthUnits = TorquePerLengthUnits.NewtonMeterPerMeter) -> string:
        """
        Get TorquePerLength unit abbreviation.
        Note! the default abbreviation for TorquePerLength is NewtonMeterPerMeter.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == TorquePerLengthUnits.NewtonMillimeterPerMeter:
            return """N·mm/m"""
        
        if unit_abbreviation == TorquePerLengthUnits.NewtonCentimeterPerMeter:
            return """N·cm/m"""
        
        if unit_abbreviation == TorquePerLengthUnits.NewtonMeterPerMeter:
            return """N·m/m"""
        
        if unit_abbreviation == TorquePerLengthUnits.PoundForceInchPerFoot:
            return """lbf·in/ft"""
        
        if unit_abbreviation == TorquePerLengthUnits.PoundForceFootPerFoot:
            return """lbf·ft/ft"""
        
        if unit_abbreviation == TorquePerLengthUnits.KilogramForceMillimeterPerMeter:
            return """kgf·mm/m"""
        
        if unit_abbreviation == TorquePerLengthUnits.KilogramForceCentimeterPerMeter:
            return """kgf·cm/m"""
        
        if unit_abbreviation == TorquePerLengthUnits.KilogramForceMeterPerMeter:
            return """kgf·m/m"""
        
        if unit_abbreviation == TorquePerLengthUnits.TonneForceMillimeterPerMeter:
            return """tf·mm/m"""
        
        if unit_abbreviation == TorquePerLengthUnits.TonneForceCentimeterPerMeter:
            return """tf·cm/m"""
        
        if unit_abbreviation == TorquePerLengthUnits.TonneForceMeterPerMeter:
            return """tf·m/m"""
        
        if unit_abbreviation == TorquePerLengthUnits.KilonewtonMillimeterPerMeter:
            return """"""
        
        if unit_abbreviation == TorquePerLengthUnits.MeganewtonMillimeterPerMeter:
            return """"""
        
        if unit_abbreviation == TorquePerLengthUnits.KilonewtonCentimeterPerMeter:
            return """"""
        
        if unit_abbreviation == TorquePerLengthUnits.MeganewtonCentimeterPerMeter:
            return """"""
        
        if unit_abbreviation == TorquePerLengthUnits.KilonewtonMeterPerMeter:
            return """"""
        
        if unit_abbreviation == TorquePerLengthUnits.MeganewtonMeterPerMeter:
            return """"""
        
        if unit_abbreviation == TorquePerLengthUnits.KilopoundForceInchPerFoot:
            return """"""
        
        if unit_abbreviation == TorquePerLengthUnits.MegapoundForceInchPerFoot:
            return """"""
        
        if unit_abbreviation == TorquePerLengthUnits.KilopoundForceFootPerFoot:
            return """"""
        
        if unit_abbreviation == TorquePerLengthUnits.MegapoundForceFootPerFoot:
            return """"""
        

    def __str__(self):
        return self.to_string()


    def __add__(self, other):
        if not isinstance(other, TorquePerLength):
            raise TypeError("unsupported operand type(s) for +: 'TorquePerLength' and '{}'".format(type(other).__name__))
        return TorquePerLength(self.__value + other.__value)


    def __mul__(self, other):
        if not isinstance(other, TorquePerLength):
            raise TypeError("unsupported operand type(s) for *: 'TorquePerLength' and '{}'".format(type(other).__name__))
        return TorquePerLength(self.__value * other.__value)


    def __sub__(self, other):
        if not isinstance(other, TorquePerLength):
            raise TypeError("unsupported operand type(s) for -: 'TorquePerLength' and '{}'".format(type(other).__name__))
        return TorquePerLength(self.__value - other.__value)


    def __truediv__(self, other):
        if not isinstance(other, TorquePerLength):
            raise TypeError("unsupported operand type(s) for /: 'TorquePerLength' and '{}'".format(type(other).__name__))
        return TorquePerLength(self.__value / other.__value)


    def __mod__(self, other):
        if not isinstance(other, TorquePerLength):
            raise TypeError("unsupported operand type(s) for %: 'TorquePerLength' and '{}'".format(type(other).__name__))
        return TorquePerLength(self.__value % other.__value)


    def __pow__(self, other):
        if not isinstance(other, TorquePerLength):
            raise TypeError("unsupported operand type(s) for **: 'TorquePerLength' and '{}'".format(type(other).__name__))
        return TorquePerLength(self.__value ** other.__value)


    def __eq__(self, other):
        if not isinstance(other, TorquePerLength):
            raise TypeError("unsupported operand type(s) for ==: 'TorquePerLength' and '{}'".format(type(other).__name__))
        return self.__value == other.__value


    def __lt__(self, other):
        if not isinstance(other, TorquePerLength):
            raise TypeError("unsupported operand type(s) for <: 'TorquePerLength' and '{}'".format(type(other).__name__))
        return self.__value < other.__value


    def __gt__(self, other):
        if not isinstance(other, TorquePerLength):
            raise TypeError("unsupported operand type(s) for >: 'TorquePerLength' and '{}'".format(type(other).__name__))
        return self.__value > other.__value


    def __le__(self, other):
        if not isinstance(other, TorquePerLength):
            raise TypeError("unsupported operand type(s) for <=: 'TorquePerLength' and '{}'".format(type(other).__name__))
        return self.__value <= other.__value


    def __ge__(self, other):
        if not isinstance(other, TorquePerLength):
            raise TypeError("unsupported operand type(s) for >=: 'TorquePerLength' and '{}'".format(type(other).__name__))
        return self.__value >= other.__value


from enum import Enum
import math
import string


class CapacitanceUnits(Enum):
        """
            CapacitanceUnits enumeration
        """
        
        Farad = 'farad'
        """
            
        """
        
        Picofarad = 'picofarad'
        """
            
        """
        
        Nanofarad = 'nanofarad'
        """
            
        """
        
        Microfarad = 'microfarad'
        """
            
        """
        
        Millifarad = 'millifarad'
        """
            
        """
        
        Kilofarad = 'kilofarad'
        """
            
        """
        
        Megafarad = 'megafarad'
        """
            
        """
        

class Capacitance:
    """
    Capacitance is the ability of a body to store an electric charge.

    Args:
        value (float): The value.
        from_unit (CapacitanceUnits): The Capacitance unit to create from, The default unit is Farad
    """
    def __init__(self, value: float, from_unit: CapacitanceUnits = CapacitanceUnits.Farad):
        if math.isnan(value):
            raise ValueError('Invalid unit: value is NaN')
        self.__value = self.__convert_to_base(value, from_unit)
        
        self.__farads = None
        
        self.__picofarads = None
        
        self.__nanofarads = None
        
        self.__microfarads = None
        
        self.__millifarads = None
        
        self.__kilofarads = None
        
        self.__megafarads = None
        

    def __convert_from_base(self, from_unit: CapacitanceUnits) -> float:
        value = self.__value
        
        if from_unit == CapacitanceUnits.Farad:
            return (value)
        
        if from_unit == CapacitanceUnits.Picofarad:
            return ((value) / 1e-12)
        
        if from_unit == CapacitanceUnits.Nanofarad:
            return ((value) / 1e-09)
        
        if from_unit == CapacitanceUnits.Microfarad:
            return ((value) / 1e-06)
        
        if from_unit == CapacitanceUnits.Millifarad:
            return ((value) / 0.001)
        
        if from_unit == CapacitanceUnits.Kilofarad:
            return ((value) / 1000.0)
        
        if from_unit == CapacitanceUnits.Megafarad:
            return ((value) / 1000000.0)
        
        return None


    def __convert_to_base(self, value: float, to_unit: CapacitanceUnits) -> float:
        
        if to_unit == CapacitanceUnits.Farad:
            return (value)
        
        if to_unit == CapacitanceUnits.Picofarad:
            return ((value) * 1e-12)
        
        if to_unit == CapacitanceUnits.Nanofarad:
            return ((value) * 1e-09)
        
        if to_unit == CapacitanceUnits.Microfarad:
            return ((value) * 1e-06)
        
        if to_unit == CapacitanceUnits.Millifarad:
            return ((value) * 0.001)
        
        if to_unit == CapacitanceUnits.Kilofarad:
            return ((value) * 1000.0)
        
        if to_unit == CapacitanceUnits.Megafarad:
            return ((value) * 1000000.0)
        
        return None


    @property
    def base_value(self) -> float:
        return self.__value

    
    @staticmethod
    def from_farads(farads: float):
        """
        Create a new instance of Capacitance from a value in farads.

        

        :param meters: The Capacitance value in farads.
        :type farads: float
        :return: A new instance of Capacitance.
        :rtype: Capacitance
        """
        return Capacitance(farads, CapacitanceUnits.Farad)

    
    @staticmethod
    def from_picofarads(picofarads: float):
        """
        Create a new instance of Capacitance from a value in picofarads.

        

        :param meters: The Capacitance value in picofarads.
        :type picofarads: float
        :return: A new instance of Capacitance.
        :rtype: Capacitance
        """
        return Capacitance(picofarads, CapacitanceUnits.Picofarad)

    
    @staticmethod
    def from_nanofarads(nanofarads: float):
        """
        Create a new instance of Capacitance from a value in nanofarads.

        

        :param meters: The Capacitance value in nanofarads.
        :type nanofarads: float
        :return: A new instance of Capacitance.
        :rtype: Capacitance
        """
        return Capacitance(nanofarads, CapacitanceUnits.Nanofarad)

    
    @staticmethod
    def from_microfarads(microfarads: float):
        """
        Create a new instance of Capacitance from a value in microfarads.

        

        :param meters: The Capacitance value in microfarads.
        :type microfarads: float
        :return: A new instance of Capacitance.
        :rtype: Capacitance
        """
        return Capacitance(microfarads, CapacitanceUnits.Microfarad)

    
    @staticmethod
    def from_millifarads(millifarads: float):
        """
        Create a new instance of Capacitance from a value in millifarads.

        

        :param meters: The Capacitance value in millifarads.
        :type millifarads: float
        :return: A new instance of Capacitance.
        :rtype: Capacitance
        """
        return Capacitance(millifarads, CapacitanceUnits.Millifarad)

    
    @staticmethod
    def from_kilofarads(kilofarads: float):
        """
        Create a new instance of Capacitance from a value in kilofarads.

        

        :param meters: The Capacitance value in kilofarads.
        :type kilofarads: float
        :return: A new instance of Capacitance.
        :rtype: Capacitance
        """
        return Capacitance(kilofarads, CapacitanceUnits.Kilofarad)

    
    @staticmethod
    def from_megafarads(megafarads: float):
        """
        Create a new instance of Capacitance from a value in megafarads.

        

        :param meters: The Capacitance value in megafarads.
        :type megafarads: float
        :return: A new instance of Capacitance.
        :rtype: Capacitance
        """
        return Capacitance(megafarads, CapacitanceUnits.Megafarad)

    
    @property
    def farads(self) -> float:
        """
        
        """
        if self.__farads != None:
            return self.__farads
        self.__farads = self.__convert_from_base(CapacitanceUnits.Farad)
        return self.__farads

    
    @property
    def picofarads(self) -> float:
        """
        
        """
        if self.__picofarads != None:
            return self.__picofarads
        self.__picofarads = self.__convert_from_base(CapacitanceUnits.Picofarad)
        return self.__picofarads

    
    @property
    def nanofarads(self) -> float:
        """
        
        """
        if self.__nanofarads != None:
            return self.__nanofarads
        self.__nanofarads = self.__convert_from_base(CapacitanceUnits.Nanofarad)
        return self.__nanofarads

    
    @property
    def microfarads(self) -> float:
        """
        
        """
        if self.__microfarads != None:
            return self.__microfarads
        self.__microfarads = self.__convert_from_base(CapacitanceUnits.Microfarad)
        return self.__microfarads

    
    @property
    def millifarads(self) -> float:
        """
        
        """
        if self.__millifarads != None:
            return self.__millifarads
        self.__millifarads = self.__convert_from_base(CapacitanceUnits.Millifarad)
        return self.__millifarads

    
    @property
    def kilofarads(self) -> float:
        """
        
        """
        if self.__kilofarads != None:
            return self.__kilofarads
        self.__kilofarads = self.__convert_from_base(CapacitanceUnits.Kilofarad)
        return self.__kilofarads

    
    @property
    def megafarads(self) -> float:
        """
        
        """
        if self.__megafarads != None:
            return self.__megafarads
        self.__megafarads = self.__convert_from_base(CapacitanceUnits.Megafarad)
        return self.__megafarads

    
    def to_string(self, unit: CapacitanceUnits = CapacitanceUnits.Farad) -> string:
        """
        Format the Capacitance to string.
        Note! the default format for Capacitance is Farad.
        To specify the unit format set the 'unit' parameter.
        """
        
        if unit == CapacitanceUnits.Farad:
            return f"""{self.farads} F"""
        
        if unit == CapacitanceUnits.Picofarad:
            return f"""{self.picofarads} """
        
        if unit == CapacitanceUnits.Nanofarad:
            return f"""{self.nanofarads} """
        
        if unit == CapacitanceUnits.Microfarad:
            return f"""{self.microfarads} """
        
        if unit == CapacitanceUnits.Millifarad:
            return f"""{self.millifarads} """
        
        if unit == CapacitanceUnits.Kilofarad:
            return f"""{self.kilofarads} """
        
        if unit == CapacitanceUnits.Megafarad:
            return f"""{self.megafarads} """
        
        return f'{self.__value}'


    def get_unit_abbreviation(self, unit_abbreviation: CapacitanceUnits = CapacitanceUnits.Farad) -> string:
        """
        Get Capacitance unit abbreviation.
        Note! the default abbreviation for Capacitance is Farad.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == CapacitanceUnits.Farad:
            return """F"""
        
        if unit_abbreviation == CapacitanceUnits.Picofarad:
            return """"""
        
        if unit_abbreviation == CapacitanceUnits.Nanofarad:
            return """"""
        
        if unit_abbreviation == CapacitanceUnits.Microfarad:
            return """"""
        
        if unit_abbreviation == CapacitanceUnits.Millifarad:
            return """"""
        
        if unit_abbreviation == CapacitanceUnits.Kilofarad:
            return """"""
        
        if unit_abbreviation == CapacitanceUnits.Megafarad:
            return """"""
        

    def __str__(self):
        return self.to_string()


    def __add__(self, other):
        if not isinstance(other, Capacitance):
            raise TypeError("unsupported operand type(s) for +: 'Capacitance' and '{}'".format(type(other).__name__))
        return Capacitance(self.__value + other.__value)


    def __mul__(self, other):
        if not isinstance(other, Capacitance):
            raise TypeError("unsupported operand type(s) for *: 'Capacitance' and '{}'".format(type(other).__name__))
        return Capacitance(self.__value * other.__value)


    def __sub__(self, other):
        if not isinstance(other, Capacitance):
            raise TypeError("unsupported operand type(s) for -: 'Capacitance' and '{}'".format(type(other).__name__))
        return Capacitance(self.__value - other.__value)


    def __truediv__(self, other):
        if not isinstance(other, Capacitance):
            raise TypeError("unsupported operand type(s) for /: 'Capacitance' and '{}'".format(type(other).__name__))
        return Capacitance(self.__value / other.__value)


    def __mod__(self, other):
        if not isinstance(other, Capacitance):
            raise TypeError("unsupported operand type(s) for %: 'Capacitance' and '{}'".format(type(other).__name__))
        return Capacitance(self.__value % other.__value)


    def __pow__(self, other):
        if not isinstance(other, Capacitance):
            raise TypeError("unsupported operand type(s) for **: 'Capacitance' and '{}'".format(type(other).__name__))
        return Capacitance(self.__value ** other.__value)


    def __eq__(self, other):
        if not isinstance(other, Capacitance):
            raise TypeError("unsupported operand type(s) for ==: 'Capacitance' and '{}'".format(type(other).__name__))
        return self.__value == other.__value


    def __lt__(self, other):
        if not isinstance(other, Capacitance):
            raise TypeError("unsupported operand type(s) for <: 'Capacitance' and '{}'".format(type(other).__name__))
        return self.__value < other.__value


    def __gt__(self, other):
        if not isinstance(other, Capacitance):
            raise TypeError("unsupported operand type(s) for >: 'Capacitance' and '{}'".format(type(other).__name__))
        return self.__value > other.__value


    def __le__(self, other):
        if not isinstance(other, Capacitance):
            raise TypeError("unsupported operand type(s) for <=: 'Capacitance' and '{}'".format(type(other).__name__))
        return self.__value <= other.__value


    def __ge__(self, other):
        if not isinstance(other, Capacitance):
            raise TypeError("unsupported operand type(s) for >=: 'Capacitance' and '{}'".format(type(other).__name__))
        return self.__value >= other.__value


from enum import Enum
import math
import string


class EnergyDensityUnits(Enum):
        """
            EnergyDensityUnits enumeration
        """
        
        JoulePerCubicMeter = 'joule_per_cubic_meter'
        """
            
        """
        
        WattHourPerCubicMeter = 'watt_hour_per_cubic_meter'
        """
            
        """
        
        KilojoulePerCubicMeter = 'kilojoule_per_cubic_meter'
        """
            
        """
        
        MegajoulePerCubicMeter = 'megajoule_per_cubic_meter'
        """
            
        """
        
        GigajoulePerCubicMeter = 'gigajoule_per_cubic_meter'
        """
            
        """
        
        TerajoulePerCubicMeter = 'terajoule_per_cubic_meter'
        """
            
        """
        
        PetajoulePerCubicMeter = 'petajoule_per_cubic_meter'
        """
            
        """
        
        KilowattHourPerCubicMeter = 'kilowatt_hour_per_cubic_meter'
        """
            
        """
        
        MegawattHourPerCubicMeter = 'megawatt_hour_per_cubic_meter'
        """
            
        """
        
        GigawattHourPerCubicMeter = 'gigawatt_hour_per_cubic_meter'
        """
            
        """
        
        TerawattHourPerCubicMeter = 'terawatt_hour_per_cubic_meter'
        """
            
        """
        
        PetawattHourPerCubicMeter = 'petawatt_hour_per_cubic_meter'
        """
            
        """
        

class EnergyDensity:
    """
    None

    Args:
        value (float): The value.
        from_unit (EnergyDensityUnits): The EnergyDensity unit to create from, The default unit is JoulePerCubicMeter
    """
    def __init__(self, value: float, from_unit: EnergyDensityUnits = EnergyDensityUnits.JoulePerCubicMeter):
        if math.isnan(value):
            raise ValueError('Invalid unit: value is NaN')
        self.__value = self.__convert_to_base(value, from_unit)
        
        self.__joules_per_cubic_meter = None
        
        self.__watt_hours_per_cubic_meter = None
        
        self.__kilojoules_per_cubic_meter = None
        
        self.__megajoules_per_cubic_meter = None
        
        self.__gigajoules_per_cubic_meter = None
        
        self.__terajoules_per_cubic_meter = None
        
        self.__petajoules_per_cubic_meter = None
        
        self.__kilowatt_hours_per_cubic_meter = None
        
        self.__megawatt_hours_per_cubic_meter = None
        
        self.__gigawatt_hours_per_cubic_meter = None
        
        self.__terawatt_hours_per_cubic_meter = None
        
        self.__petawatt_hours_per_cubic_meter = None
        

    def __convert_from_base(self, from_unit: EnergyDensityUnits) -> float:
        value = self.__value
        
        if from_unit == EnergyDensityUnits.JoulePerCubicMeter:
            return (value)
        
        if from_unit == EnergyDensityUnits.WattHourPerCubicMeter:
            return (value / 3.6e+3)
        
        if from_unit == EnergyDensityUnits.KilojoulePerCubicMeter:
            return ((value) / 1000.0)
        
        if from_unit == EnergyDensityUnits.MegajoulePerCubicMeter:
            return ((value) / 1000000.0)
        
        if from_unit == EnergyDensityUnits.GigajoulePerCubicMeter:
            return ((value) / 1000000000.0)
        
        if from_unit == EnergyDensityUnits.TerajoulePerCubicMeter:
            return ((value) / 1000000000000.0)
        
        if from_unit == EnergyDensityUnits.PetajoulePerCubicMeter:
            return ((value) / 1000000000000000.0)
        
        if from_unit == EnergyDensityUnits.KilowattHourPerCubicMeter:
            return ((value / 3.6e+3) / 1000.0)
        
        if from_unit == EnergyDensityUnits.MegawattHourPerCubicMeter:
            return ((value / 3.6e+3) / 1000000.0)
        
        if from_unit == EnergyDensityUnits.GigawattHourPerCubicMeter:
            return ((value / 3.6e+3) / 1000000000.0)
        
        if from_unit == EnergyDensityUnits.TerawattHourPerCubicMeter:
            return ((value / 3.6e+3) / 1000000000000.0)
        
        if from_unit == EnergyDensityUnits.PetawattHourPerCubicMeter:
            return ((value / 3.6e+3) / 1000000000000000.0)
        
        return None


    def __convert_to_base(self, value: float, to_unit: EnergyDensityUnits) -> float:
        
        if to_unit == EnergyDensityUnits.JoulePerCubicMeter:
            return (value)
        
        if to_unit == EnergyDensityUnits.WattHourPerCubicMeter:
            return (value * 3.6e+3)
        
        if to_unit == EnergyDensityUnits.KilojoulePerCubicMeter:
            return ((value) * 1000.0)
        
        if to_unit == EnergyDensityUnits.MegajoulePerCubicMeter:
            return ((value) * 1000000.0)
        
        if to_unit == EnergyDensityUnits.GigajoulePerCubicMeter:
            return ((value) * 1000000000.0)
        
        if to_unit == EnergyDensityUnits.TerajoulePerCubicMeter:
            return ((value) * 1000000000000.0)
        
        if to_unit == EnergyDensityUnits.PetajoulePerCubicMeter:
            return ((value) * 1000000000000000.0)
        
        if to_unit == EnergyDensityUnits.KilowattHourPerCubicMeter:
            return ((value * 3.6e+3) * 1000.0)
        
        if to_unit == EnergyDensityUnits.MegawattHourPerCubicMeter:
            return ((value * 3.6e+3) * 1000000.0)
        
        if to_unit == EnergyDensityUnits.GigawattHourPerCubicMeter:
            return ((value * 3.6e+3) * 1000000000.0)
        
        if to_unit == EnergyDensityUnits.TerawattHourPerCubicMeter:
            return ((value * 3.6e+3) * 1000000000000.0)
        
        if to_unit == EnergyDensityUnits.PetawattHourPerCubicMeter:
            return ((value * 3.6e+3) * 1000000000000000.0)
        
        return None


    @property
    def base_value(self) -> float:
        return self.__value

    
    @staticmethod
    def from_joules_per_cubic_meter(joules_per_cubic_meter: float):
        """
        Create a new instance of EnergyDensity from a value in joules_per_cubic_meter.

        

        :param meters: The EnergyDensity value in joules_per_cubic_meter.
        :type joules_per_cubic_meter: float
        :return: A new instance of EnergyDensity.
        :rtype: EnergyDensity
        """
        return EnergyDensity(joules_per_cubic_meter, EnergyDensityUnits.JoulePerCubicMeter)

    
    @staticmethod
    def from_watt_hours_per_cubic_meter(watt_hours_per_cubic_meter: float):
        """
        Create a new instance of EnergyDensity from a value in watt_hours_per_cubic_meter.

        

        :param meters: The EnergyDensity value in watt_hours_per_cubic_meter.
        :type watt_hours_per_cubic_meter: float
        :return: A new instance of EnergyDensity.
        :rtype: EnergyDensity
        """
        return EnergyDensity(watt_hours_per_cubic_meter, EnergyDensityUnits.WattHourPerCubicMeter)

    
    @staticmethod
    def from_kilojoules_per_cubic_meter(kilojoules_per_cubic_meter: float):
        """
        Create a new instance of EnergyDensity from a value in kilojoules_per_cubic_meter.

        

        :param meters: The EnergyDensity value in kilojoules_per_cubic_meter.
        :type kilojoules_per_cubic_meter: float
        :return: A new instance of EnergyDensity.
        :rtype: EnergyDensity
        """
        return EnergyDensity(kilojoules_per_cubic_meter, EnergyDensityUnits.KilojoulePerCubicMeter)

    
    @staticmethod
    def from_megajoules_per_cubic_meter(megajoules_per_cubic_meter: float):
        """
        Create a new instance of EnergyDensity from a value in megajoules_per_cubic_meter.

        

        :param meters: The EnergyDensity value in megajoules_per_cubic_meter.
        :type megajoules_per_cubic_meter: float
        :return: A new instance of EnergyDensity.
        :rtype: EnergyDensity
        """
        return EnergyDensity(megajoules_per_cubic_meter, EnergyDensityUnits.MegajoulePerCubicMeter)

    
    @staticmethod
    def from_gigajoules_per_cubic_meter(gigajoules_per_cubic_meter: float):
        """
        Create a new instance of EnergyDensity from a value in gigajoules_per_cubic_meter.

        

        :param meters: The EnergyDensity value in gigajoules_per_cubic_meter.
        :type gigajoules_per_cubic_meter: float
        :return: A new instance of EnergyDensity.
        :rtype: EnergyDensity
        """
        return EnergyDensity(gigajoules_per_cubic_meter, EnergyDensityUnits.GigajoulePerCubicMeter)

    
    @staticmethod
    def from_terajoules_per_cubic_meter(terajoules_per_cubic_meter: float):
        """
        Create a new instance of EnergyDensity from a value in terajoules_per_cubic_meter.

        

        :param meters: The EnergyDensity value in terajoules_per_cubic_meter.
        :type terajoules_per_cubic_meter: float
        :return: A new instance of EnergyDensity.
        :rtype: EnergyDensity
        """
        return EnergyDensity(terajoules_per_cubic_meter, EnergyDensityUnits.TerajoulePerCubicMeter)

    
    @staticmethod
    def from_petajoules_per_cubic_meter(petajoules_per_cubic_meter: float):
        """
        Create a new instance of EnergyDensity from a value in petajoules_per_cubic_meter.

        

        :param meters: The EnergyDensity value in petajoules_per_cubic_meter.
        :type petajoules_per_cubic_meter: float
        :return: A new instance of EnergyDensity.
        :rtype: EnergyDensity
        """
        return EnergyDensity(petajoules_per_cubic_meter, EnergyDensityUnits.PetajoulePerCubicMeter)

    
    @staticmethod
    def from_kilowatt_hours_per_cubic_meter(kilowatt_hours_per_cubic_meter: float):
        """
        Create a new instance of EnergyDensity from a value in kilowatt_hours_per_cubic_meter.

        

        :param meters: The EnergyDensity value in kilowatt_hours_per_cubic_meter.
        :type kilowatt_hours_per_cubic_meter: float
        :return: A new instance of EnergyDensity.
        :rtype: EnergyDensity
        """
        return EnergyDensity(kilowatt_hours_per_cubic_meter, EnergyDensityUnits.KilowattHourPerCubicMeter)

    
    @staticmethod
    def from_megawatt_hours_per_cubic_meter(megawatt_hours_per_cubic_meter: float):
        """
        Create a new instance of EnergyDensity from a value in megawatt_hours_per_cubic_meter.

        

        :param meters: The EnergyDensity value in megawatt_hours_per_cubic_meter.
        :type megawatt_hours_per_cubic_meter: float
        :return: A new instance of EnergyDensity.
        :rtype: EnergyDensity
        """
        return EnergyDensity(megawatt_hours_per_cubic_meter, EnergyDensityUnits.MegawattHourPerCubicMeter)

    
    @staticmethod
    def from_gigawatt_hours_per_cubic_meter(gigawatt_hours_per_cubic_meter: float):
        """
        Create a new instance of EnergyDensity from a value in gigawatt_hours_per_cubic_meter.

        

        :param meters: The EnergyDensity value in gigawatt_hours_per_cubic_meter.
        :type gigawatt_hours_per_cubic_meter: float
        :return: A new instance of EnergyDensity.
        :rtype: EnergyDensity
        """
        return EnergyDensity(gigawatt_hours_per_cubic_meter, EnergyDensityUnits.GigawattHourPerCubicMeter)

    
    @staticmethod
    def from_terawatt_hours_per_cubic_meter(terawatt_hours_per_cubic_meter: float):
        """
        Create a new instance of EnergyDensity from a value in terawatt_hours_per_cubic_meter.

        

        :param meters: The EnergyDensity value in terawatt_hours_per_cubic_meter.
        :type terawatt_hours_per_cubic_meter: float
        :return: A new instance of EnergyDensity.
        :rtype: EnergyDensity
        """
        return EnergyDensity(terawatt_hours_per_cubic_meter, EnergyDensityUnits.TerawattHourPerCubicMeter)

    
    @staticmethod
    def from_petawatt_hours_per_cubic_meter(petawatt_hours_per_cubic_meter: float):
        """
        Create a new instance of EnergyDensity from a value in petawatt_hours_per_cubic_meter.

        

        :param meters: The EnergyDensity value in petawatt_hours_per_cubic_meter.
        :type petawatt_hours_per_cubic_meter: float
        :return: A new instance of EnergyDensity.
        :rtype: EnergyDensity
        """
        return EnergyDensity(petawatt_hours_per_cubic_meter, EnergyDensityUnits.PetawattHourPerCubicMeter)

    
    @property
    def joules_per_cubic_meter(self) -> float:
        """
        
        """
        if self.__joules_per_cubic_meter != None:
            return self.__joules_per_cubic_meter
        self.__joules_per_cubic_meter = self.__convert_from_base(EnergyDensityUnits.JoulePerCubicMeter)
        return self.__joules_per_cubic_meter

    
    @property
    def watt_hours_per_cubic_meter(self) -> float:
        """
        
        """
        if self.__watt_hours_per_cubic_meter != None:
            return self.__watt_hours_per_cubic_meter
        self.__watt_hours_per_cubic_meter = self.__convert_from_base(EnergyDensityUnits.WattHourPerCubicMeter)
        return self.__watt_hours_per_cubic_meter

    
    @property
    def kilojoules_per_cubic_meter(self) -> float:
        """
        
        """
        if self.__kilojoules_per_cubic_meter != None:
            return self.__kilojoules_per_cubic_meter
        self.__kilojoules_per_cubic_meter = self.__convert_from_base(EnergyDensityUnits.KilojoulePerCubicMeter)
        return self.__kilojoules_per_cubic_meter

    
    @property
    def megajoules_per_cubic_meter(self) -> float:
        """
        
        """
        if self.__megajoules_per_cubic_meter != None:
            return self.__megajoules_per_cubic_meter
        self.__megajoules_per_cubic_meter = self.__convert_from_base(EnergyDensityUnits.MegajoulePerCubicMeter)
        return self.__megajoules_per_cubic_meter

    
    @property
    def gigajoules_per_cubic_meter(self) -> float:
        """
        
        """
        if self.__gigajoules_per_cubic_meter != None:
            return self.__gigajoules_per_cubic_meter
        self.__gigajoules_per_cubic_meter = self.__convert_from_base(EnergyDensityUnits.GigajoulePerCubicMeter)
        return self.__gigajoules_per_cubic_meter

    
    @property
    def terajoules_per_cubic_meter(self) -> float:
        """
        
        """
        if self.__terajoules_per_cubic_meter != None:
            return self.__terajoules_per_cubic_meter
        self.__terajoules_per_cubic_meter = self.__convert_from_base(EnergyDensityUnits.TerajoulePerCubicMeter)
        return self.__terajoules_per_cubic_meter

    
    @property
    def petajoules_per_cubic_meter(self) -> float:
        """
        
        """
        if self.__petajoules_per_cubic_meter != None:
            return self.__petajoules_per_cubic_meter
        self.__petajoules_per_cubic_meter = self.__convert_from_base(EnergyDensityUnits.PetajoulePerCubicMeter)
        return self.__petajoules_per_cubic_meter

    
    @property
    def kilowatt_hours_per_cubic_meter(self) -> float:
        """
        
        """
        if self.__kilowatt_hours_per_cubic_meter != None:
            return self.__kilowatt_hours_per_cubic_meter
        self.__kilowatt_hours_per_cubic_meter = self.__convert_from_base(EnergyDensityUnits.KilowattHourPerCubicMeter)
        return self.__kilowatt_hours_per_cubic_meter

    
    @property
    def megawatt_hours_per_cubic_meter(self) -> float:
        """
        
        """
        if self.__megawatt_hours_per_cubic_meter != None:
            return self.__megawatt_hours_per_cubic_meter
        self.__megawatt_hours_per_cubic_meter = self.__convert_from_base(EnergyDensityUnits.MegawattHourPerCubicMeter)
        return self.__megawatt_hours_per_cubic_meter

    
    @property
    def gigawatt_hours_per_cubic_meter(self) -> float:
        """
        
        """
        if self.__gigawatt_hours_per_cubic_meter != None:
            return self.__gigawatt_hours_per_cubic_meter
        self.__gigawatt_hours_per_cubic_meter = self.__convert_from_base(EnergyDensityUnits.GigawattHourPerCubicMeter)
        return self.__gigawatt_hours_per_cubic_meter

    
    @property
    def terawatt_hours_per_cubic_meter(self) -> float:
        """
        
        """
        if self.__terawatt_hours_per_cubic_meter != None:
            return self.__terawatt_hours_per_cubic_meter
        self.__terawatt_hours_per_cubic_meter = self.__convert_from_base(EnergyDensityUnits.TerawattHourPerCubicMeter)
        return self.__terawatt_hours_per_cubic_meter

    
    @property
    def petawatt_hours_per_cubic_meter(self) -> float:
        """
        
        """
        if self.__petawatt_hours_per_cubic_meter != None:
            return self.__petawatt_hours_per_cubic_meter
        self.__petawatt_hours_per_cubic_meter = self.__convert_from_base(EnergyDensityUnits.PetawattHourPerCubicMeter)
        return self.__petawatt_hours_per_cubic_meter

    
    def to_string(self, unit: EnergyDensityUnits = EnergyDensityUnits.JoulePerCubicMeter) -> string:
        """
        Format the EnergyDensity to string.
        Note! the default format for EnergyDensity is JoulePerCubicMeter.
        To specify the unit format set the 'unit' parameter.
        """
        
        if unit == EnergyDensityUnits.JoulePerCubicMeter:
            return f"""{self.joules_per_cubic_meter} J/m³"""
        
        if unit == EnergyDensityUnits.WattHourPerCubicMeter:
            return f"""{self.watt_hours_per_cubic_meter} Wh/m³"""
        
        if unit == EnergyDensityUnits.KilojoulePerCubicMeter:
            return f"""{self.kilojoules_per_cubic_meter} """
        
        if unit == EnergyDensityUnits.MegajoulePerCubicMeter:
            return f"""{self.megajoules_per_cubic_meter} """
        
        if unit == EnergyDensityUnits.GigajoulePerCubicMeter:
            return f"""{self.gigajoules_per_cubic_meter} """
        
        if unit == EnergyDensityUnits.TerajoulePerCubicMeter:
            return f"""{self.terajoules_per_cubic_meter} """
        
        if unit == EnergyDensityUnits.PetajoulePerCubicMeter:
            return f"""{self.petajoules_per_cubic_meter} """
        
        if unit == EnergyDensityUnits.KilowattHourPerCubicMeter:
            return f"""{self.kilowatt_hours_per_cubic_meter} """
        
        if unit == EnergyDensityUnits.MegawattHourPerCubicMeter:
            return f"""{self.megawatt_hours_per_cubic_meter} """
        
        if unit == EnergyDensityUnits.GigawattHourPerCubicMeter:
            return f"""{self.gigawatt_hours_per_cubic_meter} """
        
        if unit == EnergyDensityUnits.TerawattHourPerCubicMeter:
            return f"""{self.terawatt_hours_per_cubic_meter} """
        
        if unit == EnergyDensityUnits.PetawattHourPerCubicMeter:
            return f"""{self.petawatt_hours_per_cubic_meter} """
        
        return f'{self.__value}'


    def get_unit_abbreviation(self, unit_abbreviation: EnergyDensityUnits = EnergyDensityUnits.JoulePerCubicMeter) -> string:
        """
        Get EnergyDensity unit abbreviation.
        Note! the default abbreviation for EnergyDensity is JoulePerCubicMeter.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == EnergyDensityUnits.JoulePerCubicMeter:
            return """J/m³"""
        
        if unit_abbreviation == EnergyDensityUnits.WattHourPerCubicMeter:
            return """Wh/m³"""
        
        if unit_abbreviation == EnergyDensityUnits.KilojoulePerCubicMeter:
            return """"""
        
        if unit_abbreviation == EnergyDensityUnits.MegajoulePerCubicMeter:
            return """"""
        
        if unit_abbreviation == EnergyDensityUnits.GigajoulePerCubicMeter:
            return """"""
        
        if unit_abbreviation == EnergyDensityUnits.TerajoulePerCubicMeter:
            return """"""
        
        if unit_abbreviation == EnergyDensityUnits.PetajoulePerCubicMeter:
            return """"""
        
        if unit_abbreviation == EnergyDensityUnits.KilowattHourPerCubicMeter:
            return """"""
        
        if unit_abbreviation == EnergyDensityUnits.MegawattHourPerCubicMeter:
            return """"""
        
        if unit_abbreviation == EnergyDensityUnits.GigawattHourPerCubicMeter:
            return """"""
        
        if unit_abbreviation == EnergyDensityUnits.TerawattHourPerCubicMeter:
            return """"""
        
        if unit_abbreviation == EnergyDensityUnits.PetawattHourPerCubicMeter:
            return """"""
        

    def __str__(self):
        return self.to_string()


    def __add__(self, other):
        if not isinstance(other, EnergyDensity):
            raise TypeError("unsupported operand type(s) for +: 'EnergyDensity' and '{}'".format(type(other).__name__))
        return EnergyDensity(self.__value + other.__value)


    def __mul__(self, other):
        if not isinstance(other, EnergyDensity):
            raise TypeError("unsupported operand type(s) for *: 'EnergyDensity' and '{}'".format(type(other).__name__))
        return EnergyDensity(self.__value * other.__value)


    def __sub__(self, other):
        if not isinstance(other, EnergyDensity):
            raise TypeError("unsupported operand type(s) for -: 'EnergyDensity' and '{}'".format(type(other).__name__))
        return EnergyDensity(self.__value - other.__value)


    def __truediv__(self, other):
        if not isinstance(other, EnergyDensity):
            raise TypeError("unsupported operand type(s) for /: 'EnergyDensity' and '{}'".format(type(other).__name__))
        return EnergyDensity(self.__value / other.__value)


    def __mod__(self, other):
        if not isinstance(other, EnergyDensity):
            raise TypeError("unsupported operand type(s) for %: 'EnergyDensity' and '{}'".format(type(other).__name__))
        return EnergyDensity(self.__value % other.__value)


    def __pow__(self, other):
        if not isinstance(other, EnergyDensity):
            raise TypeError("unsupported operand type(s) for **: 'EnergyDensity' and '{}'".format(type(other).__name__))
        return EnergyDensity(self.__value ** other.__value)


    def __eq__(self, other):
        if not isinstance(other, EnergyDensity):
            raise TypeError("unsupported operand type(s) for ==: 'EnergyDensity' and '{}'".format(type(other).__name__))
        return self.__value == other.__value


    def __lt__(self, other):
        if not isinstance(other, EnergyDensity):
            raise TypeError("unsupported operand type(s) for <: 'EnergyDensity' and '{}'".format(type(other).__name__))
        return self.__value < other.__value


    def __gt__(self, other):
        if not isinstance(other, EnergyDensity):
            raise TypeError("unsupported operand type(s) for >: 'EnergyDensity' and '{}'".format(type(other).__name__))
        return self.__value > other.__value


    def __le__(self, other):
        if not isinstance(other, EnergyDensity):
            raise TypeError("unsupported operand type(s) for <=: 'EnergyDensity' and '{}'".format(type(other).__name__))
        return self.__value <= other.__value


    def __ge__(self, other):
        if not isinstance(other, EnergyDensity):
            raise TypeError("unsupported operand type(s) for >=: 'EnergyDensity' and '{}'".format(type(other).__name__))
        return self.__value >= other.__value


from enum import Enum
import math
import string


class SpeedUnits(Enum):
        """
            SpeedUnits enumeration
        """
        
        MeterPerSecond = 'meter_per_second'
        """
            
        """
        
        MeterPerMinute = 'meter_per_minute'
        """
            
        """
        
        MeterPerHour = 'meter_per_hour'
        """
            
        """
        
        FootPerSecond = 'foot_per_second'
        """
            
        """
        
        FootPerMinute = 'foot_per_minute'
        """
            
        """
        
        FootPerHour = 'foot_per_hour'
        """
            
        """
        
        UsSurveyFootPerSecond = 'us_survey_foot_per_second'
        """
            
        """
        
        UsSurveyFootPerMinute = 'us_survey_foot_per_minute'
        """
            
        """
        
        UsSurveyFootPerHour = 'us_survey_foot_per_hour'
        """
            
        """
        
        InchPerSecond = 'inch_per_second'
        """
            
        """
        
        InchPerMinute = 'inch_per_minute'
        """
            
        """
        
        InchPerHour = 'inch_per_hour'
        """
            
        """
        
        YardPerSecond = 'yard_per_second'
        """
            
        """
        
        YardPerMinute = 'yard_per_minute'
        """
            
        """
        
        YardPerHour = 'yard_per_hour'
        """
            
        """
        
        Knot = 'knot'
        """
            The knot, by definition, is a unit of speed equals to 1 nautical mile per hour, which is exactly 1852.000 metres per hour. The length of the internationally agreed nautical mile is 1852 m. The US adopted the international definition in 1954, the UK adopted the international nautical mile definition in 1970.
        """
        
        MilePerHour = 'mile_per_hour'
        """
            
        """
        
        Mach = 'mach'
        """
            
        """
        
        NanometerPerSecond = 'nanometer_per_second'
        """
            
        """
        
        MicrometerPerSecond = 'micrometer_per_second'
        """
            
        """
        
        MillimeterPerSecond = 'millimeter_per_second'
        """
            
        """
        
        CentimeterPerSecond = 'centimeter_per_second'
        """
            
        """
        
        DecimeterPerSecond = 'decimeter_per_second'
        """
            
        """
        
        KilometerPerSecond = 'kilometer_per_second'
        """
            
        """
        
        NanometerPerMinute = 'nanometer_per_minute'
        """
            
        """
        
        MicrometerPerMinute = 'micrometer_per_minute'
        """
            
        """
        
        MillimeterPerMinute = 'millimeter_per_minute'
        """
            
        """
        
        CentimeterPerMinute = 'centimeter_per_minute'
        """
            
        """
        
        DecimeterPerMinute = 'decimeter_per_minute'
        """
            
        """
        
        KilometerPerMinute = 'kilometer_per_minute'
        """
            
        """
        
        MillimeterPerHour = 'millimeter_per_hour'
        """
            
        """
        
        CentimeterPerHour = 'centimeter_per_hour'
        """
            
        """
        
        KilometerPerHour = 'kilometer_per_hour'
        """
            
        """
        

class Speed:
    """
    In everyday use and in kinematics, the speed of an object is the magnitude of its velocity (the rate of change of its position); it is thus a scalar quantity.[1] The average speed of an object in an interval of time is the distance travelled by the object divided by the duration of the interval;[2] the instantaneous speed is the limit of the average speed as the duration of the time interval approaches zero.

    Args:
        value (float): The value.
        from_unit (SpeedUnits): The Speed unit to create from, The default unit is MeterPerSecond
    """
    def __init__(self, value: float, from_unit: SpeedUnits = SpeedUnits.MeterPerSecond):
        if math.isnan(value):
            raise ValueError('Invalid unit: value is NaN')
        self.__value = self.__convert_to_base(value, from_unit)
        
        self.__meters_per_second = None
        
        self.__meters_per_minutes = None
        
        self.__meters_per_hour = None
        
        self.__feet_per_second = None
        
        self.__feet_per_minute = None
        
        self.__feet_per_hour = None
        
        self.__us_survey_feet_per_second = None
        
        self.__us_survey_feet_per_minute = None
        
        self.__us_survey_feet_per_hour = None
        
        self.__inches_per_second = None
        
        self.__inches_per_minute = None
        
        self.__inches_per_hour = None
        
        self.__yards_per_second = None
        
        self.__yards_per_minute = None
        
        self.__yards_per_hour = None
        
        self.__knots = None
        
        self.__miles_per_hour = None
        
        self.__mach = None
        
        self.__nanometers_per_second = None
        
        self.__micrometers_per_second = None
        
        self.__millimeters_per_second = None
        
        self.__centimeters_per_second = None
        
        self.__decimeters_per_second = None
        
        self.__kilometers_per_second = None
        
        self.__nanometers_per_minutes = None
        
        self.__micrometers_per_minutes = None
        
        self.__millimeters_per_minutes = None
        
        self.__centimeters_per_minutes = None
        
        self.__decimeters_per_minutes = None
        
        self.__kilometers_per_minutes = None
        
        self.__millimeters_per_hour = None
        
        self.__centimeters_per_hour = None
        
        self.__kilometers_per_hour = None
        

    def __convert_from_base(self, from_unit: SpeedUnits) -> float:
        value = self.__value
        
        if from_unit == SpeedUnits.MeterPerSecond:
            return (value)
        
        if from_unit == SpeedUnits.MeterPerMinute:
            return (value * 60)
        
        if from_unit == SpeedUnits.MeterPerHour:
            return (value * 3600)
        
        if from_unit == SpeedUnits.FootPerSecond:
            return (value / 0.3048)
        
        if from_unit == SpeedUnits.FootPerMinute:
            return (value / 0.3048 * 60)
        
        if from_unit == SpeedUnits.FootPerHour:
            return (value / 0.3048 * 3600)
        
        if from_unit == SpeedUnits.UsSurveyFootPerSecond:
            return (value * 3937 / 1200)
        
        if from_unit == SpeedUnits.UsSurveyFootPerMinute:
            return ((value * 3937 / 1200) * 60)
        
        if from_unit == SpeedUnits.UsSurveyFootPerHour:
            return ((value * 3937 / 1200) * 3600)
        
        if from_unit == SpeedUnits.InchPerSecond:
            return (value / 2.54e-2)
        
        if from_unit == SpeedUnits.InchPerMinute:
            return ((value / 2.54e-2) * 60)
        
        if from_unit == SpeedUnits.InchPerHour:
            return ((value / 2.54e-2) * 3600)
        
        if from_unit == SpeedUnits.YardPerSecond:
            return (value / 0.9144)
        
        if from_unit == SpeedUnits.YardPerMinute:
            return (value / 0.9144 * 60)
        
        if from_unit == SpeedUnits.YardPerHour:
            return (value / 0.9144 * 3600)
        
        if from_unit == SpeedUnits.Knot:
            return (value / (1852.0 / 3600.0))
        
        if from_unit == SpeedUnits.MilePerHour:
            return (value / 0.44704)
        
        if from_unit == SpeedUnits.Mach:
            return (value / 340.29)
        
        if from_unit == SpeedUnits.NanometerPerSecond:
            return ((value) / 1e-09)
        
        if from_unit == SpeedUnits.MicrometerPerSecond:
            return ((value) / 1e-06)
        
        if from_unit == SpeedUnits.MillimeterPerSecond:
            return ((value) / 0.001)
        
        if from_unit == SpeedUnits.CentimeterPerSecond:
            return ((value) / 0.01)
        
        if from_unit == SpeedUnits.DecimeterPerSecond:
            return ((value) / 0.1)
        
        if from_unit == SpeedUnits.KilometerPerSecond:
            return ((value) / 1000.0)
        
        if from_unit == SpeedUnits.NanometerPerMinute:
            return ((value * 60) / 1e-09)
        
        if from_unit == SpeedUnits.MicrometerPerMinute:
            return ((value * 60) / 1e-06)
        
        if from_unit == SpeedUnits.MillimeterPerMinute:
            return ((value * 60) / 0.001)
        
        if from_unit == SpeedUnits.CentimeterPerMinute:
            return ((value * 60) / 0.01)
        
        if from_unit == SpeedUnits.DecimeterPerMinute:
            return ((value * 60) / 0.1)
        
        if from_unit == SpeedUnits.KilometerPerMinute:
            return ((value * 60) / 1000.0)
        
        if from_unit == SpeedUnits.MillimeterPerHour:
            return ((value * 3600) / 0.001)
        
        if from_unit == SpeedUnits.CentimeterPerHour:
            return ((value * 3600) / 0.01)
        
        if from_unit == SpeedUnits.KilometerPerHour:
            return ((value * 3600) / 1000.0)
        
        return None


    def __convert_to_base(self, value: float, to_unit: SpeedUnits) -> float:
        
        if to_unit == SpeedUnits.MeterPerSecond:
            return (value)
        
        if to_unit == SpeedUnits.MeterPerMinute:
            return (value / 60)
        
        if to_unit == SpeedUnits.MeterPerHour:
            return (value / 3600)
        
        if to_unit == SpeedUnits.FootPerSecond:
            return (value * 0.3048)
        
        if to_unit == SpeedUnits.FootPerMinute:
            return (value * 0.3048 / 60)
        
        if to_unit == SpeedUnits.FootPerHour:
            return (value * 0.3048 / 3600)
        
        if to_unit == SpeedUnits.UsSurveyFootPerSecond:
            return (value * 1200 / 3937)
        
        if to_unit == SpeedUnits.UsSurveyFootPerMinute:
            return ((value * 1200 / 3937) / 60)
        
        if to_unit == SpeedUnits.UsSurveyFootPerHour:
            return ((value * 1200 / 3937) / 3600)
        
        if to_unit == SpeedUnits.InchPerSecond:
            return (value * 2.54e-2)
        
        if to_unit == SpeedUnits.InchPerMinute:
            return ((value / 60) * 2.54e-2)
        
        if to_unit == SpeedUnits.InchPerHour:
            return ((value / 3600) * 2.54e-2)
        
        if to_unit == SpeedUnits.YardPerSecond:
            return (value * 0.9144)
        
        if to_unit == SpeedUnits.YardPerMinute:
            return (value * 0.9144 / 60)
        
        if to_unit == SpeedUnits.YardPerHour:
            return (value * 0.9144 / 3600)
        
        if to_unit == SpeedUnits.Knot:
            return (value * (1852.0 / 3600.0))
        
        if to_unit == SpeedUnits.MilePerHour:
            return (value * 0.44704)
        
        if to_unit == SpeedUnits.Mach:
            return (value * 340.29)
        
        if to_unit == SpeedUnits.NanometerPerSecond:
            return ((value) * 1e-09)
        
        if to_unit == SpeedUnits.MicrometerPerSecond:
            return ((value) * 1e-06)
        
        if to_unit == SpeedUnits.MillimeterPerSecond:
            return ((value) * 0.001)
        
        if to_unit == SpeedUnits.CentimeterPerSecond:
            return ((value) * 0.01)
        
        if to_unit == SpeedUnits.DecimeterPerSecond:
            return ((value) * 0.1)
        
        if to_unit == SpeedUnits.KilometerPerSecond:
            return ((value) * 1000.0)
        
        if to_unit == SpeedUnits.NanometerPerMinute:
            return ((value / 60) * 1e-09)
        
        if to_unit == SpeedUnits.MicrometerPerMinute:
            return ((value / 60) * 1e-06)
        
        if to_unit == SpeedUnits.MillimeterPerMinute:
            return ((value / 60) * 0.001)
        
        if to_unit == SpeedUnits.CentimeterPerMinute:
            return ((value / 60) * 0.01)
        
        if to_unit == SpeedUnits.DecimeterPerMinute:
            return ((value / 60) * 0.1)
        
        if to_unit == SpeedUnits.KilometerPerMinute:
            return ((value / 60) * 1000.0)
        
        if to_unit == SpeedUnits.MillimeterPerHour:
            return ((value / 3600) * 0.001)
        
        if to_unit == SpeedUnits.CentimeterPerHour:
            return ((value / 3600) * 0.01)
        
        if to_unit == SpeedUnits.KilometerPerHour:
            return ((value / 3600) * 1000.0)
        
        return None


    @property
    def base_value(self) -> float:
        return self.__value

    
    @staticmethod
    def from_meters_per_second(meters_per_second: float):
        """
        Create a new instance of Speed from a value in meters_per_second.

        

        :param meters: The Speed value in meters_per_second.
        :type meters_per_second: float
        :return: A new instance of Speed.
        :rtype: Speed
        """
        return Speed(meters_per_second, SpeedUnits.MeterPerSecond)

    
    @staticmethod
    def from_meters_per_minutes(meters_per_minutes: float):
        """
        Create a new instance of Speed from a value in meters_per_minutes.

        

        :param meters: The Speed value in meters_per_minutes.
        :type meters_per_minutes: float
        :return: A new instance of Speed.
        :rtype: Speed
        """
        return Speed(meters_per_minutes, SpeedUnits.MeterPerMinute)

    
    @staticmethod
    def from_meters_per_hour(meters_per_hour: float):
        """
        Create a new instance of Speed from a value in meters_per_hour.

        

        :param meters: The Speed value in meters_per_hour.
        :type meters_per_hour: float
        :return: A new instance of Speed.
        :rtype: Speed
        """
        return Speed(meters_per_hour, SpeedUnits.MeterPerHour)

    
    @staticmethod
    def from_feet_per_second(feet_per_second: float):
        """
        Create a new instance of Speed from a value in feet_per_second.

        

        :param meters: The Speed value in feet_per_second.
        :type feet_per_second: float
        :return: A new instance of Speed.
        :rtype: Speed
        """
        return Speed(feet_per_second, SpeedUnits.FootPerSecond)

    
    @staticmethod
    def from_feet_per_minute(feet_per_minute: float):
        """
        Create a new instance of Speed from a value in feet_per_minute.

        

        :param meters: The Speed value in feet_per_minute.
        :type feet_per_minute: float
        :return: A new instance of Speed.
        :rtype: Speed
        """
        return Speed(feet_per_minute, SpeedUnits.FootPerMinute)

    
    @staticmethod
    def from_feet_per_hour(feet_per_hour: float):
        """
        Create a new instance of Speed from a value in feet_per_hour.

        

        :param meters: The Speed value in feet_per_hour.
        :type feet_per_hour: float
        :return: A new instance of Speed.
        :rtype: Speed
        """
        return Speed(feet_per_hour, SpeedUnits.FootPerHour)

    
    @staticmethod
    def from_us_survey_feet_per_second(us_survey_feet_per_second: float):
        """
        Create a new instance of Speed from a value in us_survey_feet_per_second.

        

        :param meters: The Speed value in us_survey_feet_per_second.
        :type us_survey_feet_per_second: float
        :return: A new instance of Speed.
        :rtype: Speed
        """
        return Speed(us_survey_feet_per_second, SpeedUnits.UsSurveyFootPerSecond)

    
    @staticmethod
    def from_us_survey_feet_per_minute(us_survey_feet_per_minute: float):
        """
        Create a new instance of Speed from a value in us_survey_feet_per_minute.

        

        :param meters: The Speed value in us_survey_feet_per_minute.
        :type us_survey_feet_per_minute: float
        :return: A new instance of Speed.
        :rtype: Speed
        """
        return Speed(us_survey_feet_per_minute, SpeedUnits.UsSurveyFootPerMinute)

    
    @staticmethod
    def from_us_survey_feet_per_hour(us_survey_feet_per_hour: float):
        """
        Create a new instance of Speed from a value in us_survey_feet_per_hour.

        

        :param meters: The Speed value in us_survey_feet_per_hour.
        :type us_survey_feet_per_hour: float
        :return: A new instance of Speed.
        :rtype: Speed
        """
        return Speed(us_survey_feet_per_hour, SpeedUnits.UsSurveyFootPerHour)

    
    @staticmethod
    def from_inches_per_second(inches_per_second: float):
        """
        Create a new instance of Speed from a value in inches_per_second.

        

        :param meters: The Speed value in inches_per_second.
        :type inches_per_second: float
        :return: A new instance of Speed.
        :rtype: Speed
        """
        return Speed(inches_per_second, SpeedUnits.InchPerSecond)

    
    @staticmethod
    def from_inches_per_minute(inches_per_minute: float):
        """
        Create a new instance of Speed from a value in inches_per_minute.

        

        :param meters: The Speed value in inches_per_minute.
        :type inches_per_minute: float
        :return: A new instance of Speed.
        :rtype: Speed
        """
        return Speed(inches_per_minute, SpeedUnits.InchPerMinute)

    
    @staticmethod
    def from_inches_per_hour(inches_per_hour: float):
        """
        Create a new instance of Speed from a value in inches_per_hour.

        

        :param meters: The Speed value in inches_per_hour.
        :type inches_per_hour: float
        :return: A new instance of Speed.
        :rtype: Speed
        """
        return Speed(inches_per_hour, SpeedUnits.InchPerHour)

    
    @staticmethod
    def from_yards_per_second(yards_per_second: float):
        """
        Create a new instance of Speed from a value in yards_per_second.

        

        :param meters: The Speed value in yards_per_second.
        :type yards_per_second: float
        :return: A new instance of Speed.
        :rtype: Speed
        """
        return Speed(yards_per_second, SpeedUnits.YardPerSecond)

    
    @staticmethod
    def from_yards_per_minute(yards_per_minute: float):
        """
        Create a new instance of Speed from a value in yards_per_minute.

        

        :param meters: The Speed value in yards_per_minute.
        :type yards_per_minute: float
        :return: A new instance of Speed.
        :rtype: Speed
        """
        return Speed(yards_per_minute, SpeedUnits.YardPerMinute)

    
    @staticmethod
    def from_yards_per_hour(yards_per_hour: float):
        """
        Create a new instance of Speed from a value in yards_per_hour.

        

        :param meters: The Speed value in yards_per_hour.
        :type yards_per_hour: float
        :return: A new instance of Speed.
        :rtype: Speed
        """
        return Speed(yards_per_hour, SpeedUnits.YardPerHour)

    
    @staticmethod
    def from_knots(knots: float):
        """
        Create a new instance of Speed from a value in knots.

        The knot, by definition, is a unit of speed equals to 1 nautical mile per hour, which is exactly 1852.000 metres per hour. The length of the internationally agreed nautical mile is 1852 m. The US adopted the international definition in 1954, the UK adopted the international nautical mile definition in 1970.

        :param meters: The Speed value in knots.
        :type knots: float
        :return: A new instance of Speed.
        :rtype: Speed
        """
        return Speed(knots, SpeedUnits.Knot)

    
    @staticmethod
    def from_miles_per_hour(miles_per_hour: float):
        """
        Create a new instance of Speed from a value in miles_per_hour.

        

        :param meters: The Speed value in miles_per_hour.
        :type miles_per_hour: float
        :return: A new instance of Speed.
        :rtype: Speed
        """
        return Speed(miles_per_hour, SpeedUnits.MilePerHour)

    
    @staticmethod
    def from_mach(mach: float):
        """
        Create a new instance of Speed from a value in mach.

        

        :param meters: The Speed value in mach.
        :type mach: float
        :return: A new instance of Speed.
        :rtype: Speed
        """
        return Speed(mach, SpeedUnits.Mach)

    
    @staticmethod
    def from_nanometers_per_second(nanometers_per_second: float):
        """
        Create a new instance of Speed from a value in nanometers_per_second.

        

        :param meters: The Speed value in nanometers_per_second.
        :type nanometers_per_second: float
        :return: A new instance of Speed.
        :rtype: Speed
        """
        return Speed(nanometers_per_second, SpeedUnits.NanometerPerSecond)

    
    @staticmethod
    def from_micrometers_per_second(micrometers_per_second: float):
        """
        Create a new instance of Speed from a value in micrometers_per_second.

        

        :param meters: The Speed value in micrometers_per_second.
        :type micrometers_per_second: float
        :return: A new instance of Speed.
        :rtype: Speed
        """
        return Speed(micrometers_per_second, SpeedUnits.MicrometerPerSecond)

    
    @staticmethod
    def from_millimeters_per_second(millimeters_per_second: float):
        """
        Create a new instance of Speed from a value in millimeters_per_second.

        

        :param meters: The Speed value in millimeters_per_second.
        :type millimeters_per_second: float
        :return: A new instance of Speed.
        :rtype: Speed
        """
        return Speed(millimeters_per_second, SpeedUnits.MillimeterPerSecond)

    
    @staticmethod
    def from_centimeters_per_second(centimeters_per_second: float):
        """
        Create a new instance of Speed from a value in centimeters_per_second.

        

        :param meters: The Speed value in centimeters_per_second.
        :type centimeters_per_second: float
        :return: A new instance of Speed.
        :rtype: Speed
        """
        return Speed(centimeters_per_second, SpeedUnits.CentimeterPerSecond)

    
    @staticmethod
    def from_decimeters_per_second(decimeters_per_second: float):
        """
        Create a new instance of Speed from a value in decimeters_per_second.

        

        :param meters: The Speed value in decimeters_per_second.
        :type decimeters_per_second: float
        :return: A new instance of Speed.
        :rtype: Speed
        """
        return Speed(decimeters_per_second, SpeedUnits.DecimeterPerSecond)

    
    @staticmethod
    def from_kilometers_per_second(kilometers_per_second: float):
        """
        Create a new instance of Speed from a value in kilometers_per_second.

        

        :param meters: The Speed value in kilometers_per_second.
        :type kilometers_per_second: float
        :return: A new instance of Speed.
        :rtype: Speed
        """
        return Speed(kilometers_per_second, SpeedUnits.KilometerPerSecond)

    
    @staticmethod
    def from_nanometers_per_minutes(nanometers_per_minutes: float):
        """
        Create a new instance of Speed from a value in nanometers_per_minutes.

        

        :param meters: The Speed value in nanometers_per_minutes.
        :type nanometers_per_minutes: float
        :return: A new instance of Speed.
        :rtype: Speed
        """
        return Speed(nanometers_per_minutes, SpeedUnits.NanometerPerMinute)

    
    @staticmethod
    def from_micrometers_per_minutes(micrometers_per_minutes: float):
        """
        Create a new instance of Speed from a value in micrometers_per_minutes.

        

        :param meters: The Speed value in micrometers_per_minutes.
        :type micrometers_per_minutes: float
        :return: A new instance of Speed.
        :rtype: Speed
        """
        return Speed(micrometers_per_minutes, SpeedUnits.MicrometerPerMinute)

    
    @staticmethod
    def from_millimeters_per_minutes(millimeters_per_minutes: float):
        """
        Create a new instance of Speed from a value in millimeters_per_minutes.

        

        :param meters: The Speed value in millimeters_per_minutes.
        :type millimeters_per_minutes: float
        :return: A new instance of Speed.
        :rtype: Speed
        """
        return Speed(millimeters_per_minutes, SpeedUnits.MillimeterPerMinute)

    
    @staticmethod
    def from_centimeters_per_minutes(centimeters_per_minutes: float):
        """
        Create a new instance of Speed from a value in centimeters_per_minutes.

        

        :param meters: The Speed value in centimeters_per_minutes.
        :type centimeters_per_minutes: float
        :return: A new instance of Speed.
        :rtype: Speed
        """
        return Speed(centimeters_per_minutes, SpeedUnits.CentimeterPerMinute)

    
    @staticmethod
    def from_decimeters_per_minutes(decimeters_per_minutes: float):
        """
        Create a new instance of Speed from a value in decimeters_per_minutes.

        

        :param meters: The Speed value in decimeters_per_minutes.
        :type decimeters_per_minutes: float
        :return: A new instance of Speed.
        :rtype: Speed
        """
        return Speed(decimeters_per_minutes, SpeedUnits.DecimeterPerMinute)

    
    @staticmethod
    def from_kilometers_per_minutes(kilometers_per_minutes: float):
        """
        Create a new instance of Speed from a value in kilometers_per_minutes.

        

        :param meters: The Speed value in kilometers_per_minutes.
        :type kilometers_per_minutes: float
        :return: A new instance of Speed.
        :rtype: Speed
        """
        return Speed(kilometers_per_minutes, SpeedUnits.KilometerPerMinute)

    
    @staticmethod
    def from_millimeters_per_hour(millimeters_per_hour: float):
        """
        Create a new instance of Speed from a value in millimeters_per_hour.

        

        :param meters: The Speed value in millimeters_per_hour.
        :type millimeters_per_hour: float
        :return: A new instance of Speed.
        :rtype: Speed
        """
        return Speed(millimeters_per_hour, SpeedUnits.MillimeterPerHour)

    
    @staticmethod
    def from_centimeters_per_hour(centimeters_per_hour: float):
        """
        Create a new instance of Speed from a value in centimeters_per_hour.

        

        :param meters: The Speed value in centimeters_per_hour.
        :type centimeters_per_hour: float
        :return: A new instance of Speed.
        :rtype: Speed
        """
        return Speed(centimeters_per_hour, SpeedUnits.CentimeterPerHour)

    
    @staticmethod
    def from_kilometers_per_hour(kilometers_per_hour: float):
        """
        Create a new instance of Speed from a value in kilometers_per_hour.

        

        :param meters: The Speed value in kilometers_per_hour.
        :type kilometers_per_hour: float
        :return: A new instance of Speed.
        :rtype: Speed
        """
        return Speed(kilometers_per_hour, SpeedUnits.KilometerPerHour)

    
    @property
    def meters_per_second(self) -> float:
        """
        
        """
        if self.__meters_per_second != None:
            return self.__meters_per_second
        self.__meters_per_second = self.__convert_from_base(SpeedUnits.MeterPerSecond)
        return self.__meters_per_second

    
    @property
    def meters_per_minutes(self) -> float:
        """
        
        """
        if self.__meters_per_minutes != None:
            return self.__meters_per_minutes
        self.__meters_per_minutes = self.__convert_from_base(SpeedUnits.MeterPerMinute)
        return self.__meters_per_minutes

    
    @property
    def meters_per_hour(self) -> float:
        """
        
        """
        if self.__meters_per_hour != None:
            return self.__meters_per_hour
        self.__meters_per_hour = self.__convert_from_base(SpeedUnits.MeterPerHour)
        return self.__meters_per_hour

    
    @property
    def feet_per_second(self) -> float:
        """
        
        """
        if self.__feet_per_second != None:
            return self.__feet_per_second
        self.__feet_per_second = self.__convert_from_base(SpeedUnits.FootPerSecond)
        return self.__feet_per_second

    
    @property
    def feet_per_minute(self) -> float:
        """
        
        """
        if self.__feet_per_minute != None:
            return self.__feet_per_minute
        self.__feet_per_minute = self.__convert_from_base(SpeedUnits.FootPerMinute)
        return self.__feet_per_minute

    
    @property
    def feet_per_hour(self) -> float:
        """
        
        """
        if self.__feet_per_hour != None:
            return self.__feet_per_hour
        self.__feet_per_hour = self.__convert_from_base(SpeedUnits.FootPerHour)
        return self.__feet_per_hour

    
    @property
    def us_survey_feet_per_second(self) -> float:
        """
        
        """
        if self.__us_survey_feet_per_second != None:
            return self.__us_survey_feet_per_second
        self.__us_survey_feet_per_second = self.__convert_from_base(SpeedUnits.UsSurveyFootPerSecond)
        return self.__us_survey_feet_per_second

    
    @property
    def us_survey_feet_per_minute(self) -> float:
        """
        
        """
        if self.__us_survey_feet_per_minute != None:
            return self.__us_survey_feet_per_minute
        self.__us_survey_feet_per_minute = self.__convert_from_base(SpeedUnits.UsSurveyFootPerMinute)
        return self.__us_survey_feet_per_minute

    
    @property
    def us_survey_feet_per_hour(self) -> float:
        """
        
        """
        if self.__us_survey_feet_per_hour != None:
            return self.__us_survey_feet_per_hour
        self.__us_survey_feet_per_hour = self.__convert_from_base(SpeedUnits.UsSurveyFootPerHour)
        return self.__us_survey_feet_per_hour

    
    @property
    def inches_per_second(self) -> float:
        """
        
        """
        if self.__inches_per_second != None:
            return self.__inches_per_second
        self.__inches_per_second = self.__convert_from_base(SpeedUnits.InchPerSecond)
        return self.__inches_per_second

    
    @property
    def inches_per_minute(self) -> float:
        """
        
        """
        if self.__inches_per_minute != None:
            return self.__inches_per_minute
        self.__inches_per_minute = self.__convert_from_base(SpeedUnits.InchPerMinute)
        return self.__inches_per_minute

    
    @property
    def inches_per_hour(self) -> float:
        """
        
        """
        if self.__inches_per_hour != None:
            return self.__inches_per_hour
        self.__inches_per_hour = self.__convert_from_base(SpeedUnits.InchPerHour)
        return self.__inches_per_hour

    
    @property
    def yards_per_second(self) -> float:
        """
        
        """
        if self.__yards_per_second != None:
            return self.__yards_per_second
        self.__yards_per_second = self.__convert_from_base(SpeedUnits.YardPerSecond)
        return self.__yards_per_second

    
    @property
    def yards_per_minute(self) -> float:
        """
        
        """
        if self.__yards_per_minute != None:
            return self.__yards_per_minute
        self.__yards_per_minute = self.__convert_from_base(SpeedUnits.YardPerMinute)
        return self.__yards_per_minute

    
    @property
    def yards_per_hour(self) -> float:
        """
        
        """
        if self.__yards_per_hour != None:
            return self.__yards_per_hour
        self.__yards_per_hour = self.__convert_from_base(SpeedUnits.YardPerHour)
        return self.__yards_per_hour

    
    @property
    def knots(self) -> float:
        """
        The knot, by definition, is a unit of speed equals to 1 nautical mile per hour, which is exactly 1852.000 metres per hour. The length of the internationally agreed nautical mile is 1852 m. The US adopted the international definition in 1954, the UK adopted the international nautical mile definition in 1970.
        """
        if self.__knots != None:
            return self.__knots
        self.__knots = self.__convert_from_base(SpeedUnits.Knot)
        return self.__knots

    
    @property
    def miles_per_hour(self) -> float:
        """
        
        """
        if self.__miles_per_hour != None:
            return self.__miles_per_hour
        self.__miles_per_hour = self.__convert_from_base(SpeedUnits.MilePerHour)
        return self.__miles_per_hour

    
    @property
    def mach(self) -> float:
        """
        
        """
        if self.__mach != None:
            return self.__mach
        self.__mach = self.__convert_from_base(SpeedUnits.Mach)
        return self.__mach

    
    @property
    def nanometers_per_second(self) -> float:
        """
        
        """
        if self.__nanometers_per_second != None:
            return self.__nanometers_per_second
        self.__nanometers_per_second = self.__convert_from_base(SpeedUnits.NanometerPerSecond)
        return self.__nanometers_per_second

    
    @property
    def micrometers_per_second(self) -> float:
        """
        
        """
        if self.__micrometers_per_second != None:
            return self.__micrometers_per_second
        self.__micrometers_per_second = self.__convert_from_base(SpeedUnits.MicrometerPerSecond)
        return self.__micrometers_per_second

    
    @property
    def millimeters_per_second(self) -> float:
        """
        
        """
        if self.__millimeters_per_second != None:
            return self.__millimeters_per_second
        self.__millimeters_per_second = self.__convert_from_base(SpeedUnits.MillimeterPerSecond)
        return self.__millimeters_per_second

    
    @property
    def centimeters_per_second(self) -> float:
        """
        
        """
        if self.__centimeters_per_second != None:
            return self.__centimeters_per_second
        self.__centimeters_per_second = self.__convert_from_base(SpeedUnits.CentimeterPerSecond)
        return self.__centimeters_per_second

    
    @property
    def decimeters_per_second(self) -> float:
        """
        
        """
        if self.__decimeters_per_second != None:
            return self.__decimeters_per_second
        self.__decimeters_per_second = self.__convert_from_base(SpeedUnits.DecimeterPerSecond)
        return self.__decimeters_per_second

    
    @property
    def kilometers_per_second(self) -> float:
        """
        
        """
        if self.__kilometers_per_second != None:
            return self.__kilometers_per_second
        self.__kilometers_per_second = self.__convert_from_base(SpeedUnits.KilometerPerSecond)
        return self.__kilometers_per_second

    
    @property
    def nanometers_per_minutes(self) -> float:
        """
        
        """
        if self.__nanometers_per_minutes != None:
            return self.__nanometers_per_minutes
        self.__nanometers_per_minutes = self.__convert_from_base(SpeedUnits.NanometerPerMinute)
        return self.__nanometers_per_minutes

    
    @property
    def micrometers_per_minutes(self) -> float:
        """
        
        """
        if self.__micrometers_per_minutes != None:
            return self.__micrometers_per_minutes
        self.__micrometers_per_minutes = self.__convert_from_base(SpeedUnits.MicrometerPerMinute)
        return self.__micrometers_per_minutes

    
    @property
    def millimeters_per_minutes(self) -> float:
        """
        
        """
        if self.__millimeters_per_minutes != None:
            return self.__millimeters_per_minutes
        self.__millimeters_per_minutes = self.__convert_from_base(SpeedUnits.MillimeterPerMinute)
        return self.__millimeters_per_minutes

    
    @property
    def centimeters_per_minutes(self) -> float:
        """
        
        """
        if self.__centimeters_per_minutes != None:
            return self.__centimeters_per_minutes
        self.__centimeters_per_minutes = self.__convert_from_base(SpeedUnits.CentimeterPerMinute)
        return self.__centimeters_per_minutes

    
    @property
    def decimeters_per_minutes(self) -> float:
        """
        
        """
        if self.__decimeters_per_minutes != None:
            return self.__decimeters_per_minutes
        self.__decimeters_per_minutes = self.__convert_from_base(SpeedUnits.DecimeterPerMinute)
        return self.__decimeters_per_minutes

    
    @property
    def kilometers_per_minutes(self) -> float:
        """
        
        """
        if self.__kilometers_per_minutes != None:
            return self.__kilometers_per_minutes
        self.__kilometers_per_minutes = self.__convert_from_base(SpeedUnits.KilometerPerMinute)
        return self.__kilometers_per_minutes

    
    @property
    def millimeters_per_hour(self) -> float:
        """
        
        """
        if self.__millimeters_per_hour != None:
            return self.__millimeters_per_hour
        self.__millimeters_per_hour = self.__convert_from_base(SpeedUnits.MillimeterPerHour)
        return self.__millimeters_per_hour

    
    @property
    def centimeters_per_hour(self) -> float:
        """
        
        """
        if self.__centimeters_per_hour != None:
            return self.__centimeters_per_hour
        self.__centimeters_per_hour = self.__convert_from_base(SpeedUnits.CentimeterPerHour)
        return self.__centimeters_per_hour

    
    @property
    def kilometers_per_hour(self) -> float:
        """
        
        """
        if self.__kilometers_per_hour != None:
            return self.__kilometers_per_hour
        self.__kilometers_per_hour = self.__convert_from_base(SpeedUnits.KilometerPerHour)
        return self.__kilometers_per_hour

    
    def to_string(self, unit: SpeedUnits = SpeedUnits.MeterPerSecond) -> string:
        """
        Format the Speed to string.
        Note! the default format for Speed is MeterPerSecond.
        To specify the unit format set the 'unit' parameter.
        """
        
        if unit == SpeedUnits.MeterPerSecond:
            return f"""{self.meters_per_second} m/s"""
        
        if unit == SpeedUnits.MeterPerMinute:
            return f"""{self.meters_per_minutes} m/min"""
        
        if unit == SpeedUnits.MeterPerHour:
            return f"""{self.meters_per_hour} m/h"""
        
        if unit == SpeedUnits.FootPerSecond:
            return f"""{self.feet_per_second} ft/s"""
        
        if unit == SpeedUnits.FootPerMinute:
            return f"""{self.feet_per_minute} ft/min"""
        
        if unit == SpeedUnits.FootPerHour:
            return f"""{self.feet_per_hour} ft/h"""
        
        if unit == SpeedUnits.UsSurveyFootPerSecond:
            return f"""{self.us_survey_feet_per_second} ftUS/s"""
        
        if unit == SpeedUnits.UsSurveyFootPerMinute:
            return f"""{self.us_survey_feet_per_minute} ftUS/min"""
        
        if unit == SpeedUnits.UsSurveyFootPerHour:
            return f"""{self.us_survey_feet_per_hour} ftUS/h"""
        
        if unit == SpeedUnits.InchPerSecond:
            return f"""{self.inches_per_second} in/s"""
        
        if unit == SpeedUnits.InchPerMinute:
            return f"""{self.inches_per_minute} in/min"""
        
        if unit == SpeedUnits.InchPerHour:
            return f"""{self.inches_per_hour} in/h"""
        
        if unit == SpeedUnits.YardPerSecond:
            return f"""{self.yards_per_second} yd/s"""
        
        if unit == SpeedUnits.YardPerMinute:
            return f"""{self.yards_per_minute} yd/min"""
        
        if unit == SpeedUnits.YardPerHour:
            return f"""{self.yards_per_hour} yd/h"""
        
        if unit == SpeedUnits.Knot:
            return f"""{self.knots} kn"""
        
        if unit == SpeedUnits.MilePerHour:
            return f"""{self.miles_per_hour} mph"""
        
        if unit == SpeedUnits.Mach:
            return f"""{self.mach} M"""
        
        if unit == SpeedUnits.NanometerPerSecond:
            return f"""{self.nanometers_per_second} """
        
        if unit == SpeedUnits.MicrometerPerSecond:
            return f"""{self.micrometers_per_second} """
        
        if unit == SpeedUnits.MillimeterPerSecond:
            return f"""{self.millimeters_per_second} """
        
        if unit == SpeedUnits.CentimeterPerSecond:
            return f"""{self.centimeters_per_second} """
        
        if unit == SpeedUnits.DecimeterPerSecond:
            return f"""{self.decimeters_per_second} """
        
        if unit == SpeedUnits.KilometerPerSecond:
            return f"""{self.kilometers_per_second} """
        
        if unit == SpeedUnits.NanometerPerMinute:
            return f"""{self.nanometers_per_minutes} """
        
        if unit == SpeedUnits.MicrometerPerMinute:
            return f"""{self.micrometers_per_minutes} """
        
        if unit == SpeedUnits.MillimeterPerMinute:
            return f"""{self.millimeters_per_minutes} """
        
        if unit == SpeedUnits.CentimeterPerMinute:
            return f"""{self.centimeters_per_minutes} """
        
        if unit == SpeedUnits.DecimeterPerMinute:
            return f"""{self.decimeters_per_minutes} """
        
        if unit == SpeedUnits.KilometerPerMinute:
            return f"""{self.kilometers_per_minutes} """
        
        if unit == SpeedUnits.MillimeterPerHour:
            return f"""{self.millimeters_per_hour} """
        
        if unit == SpeedUnits.CentimeterPerHour:
            return f"""{self.centimeters_per_hour} """
        
        if unit == SpeedUnits.KilometerPerHour:
            return f"""{self.kilometers_per_hour} """
        
        return f'{self.__value}'


    def get_unit_abbreviation(self, unit_abbreviation: SpeedUnits = SpeedUnits.MeterPerSecond) -> string:
        """
        Get Speed unit abbreviation.
        Note! the default abbreviation for Speed is MeterPerSecond.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == SpeedUnits.MeterPerSecond:
            return """m/s"""
        
        if unit_abbreviation == SpeedUnits.MeterPerMinute:
            return """m/min"""
        
        if unit_abbreviation == SpeedUnits.MeterPerHour:
            return """m/h"""
        
        if unit_abbreviation == SpeedUnits.FootPerSecond:
            return """ft/s"""
        
        if unit_abbreviation == SpeedUnits.FootPerMinute:
            return """ft/min"""
        
        if unit_abbreviation == SpeedUnits.FootPerHour:
            return """ft/h"""
        
        if unit_abbreviation == SpeedUnits.UsSurveyFootPerSecond:
            return """ftUS/s"""
        
        if unit_abbreviation == SpeedUnits.UsSurveyFootPerMinute:
            return """ftUS/min"""
        
        if unit_abbreviation == SpeedUnits.UsSurveyFootPerHour:
            return """ftUS/h"""
        
        if unit_abbreviation == SpeedUnits.InchPerSecond:
            return """in/s"""
        
        if unit_abbreviation == SpeedUnits.InchPerMinute:
            return """in/min"""
        
        if unit_abbreviation == SpeedUnits.InchPerHour:
            return """in/h"""
        
        if unit_abbreviation == SpeedUnits.YardPerSecond:
            return """yd/s"""
        
        if unit_abbreviation == SpeedUnits.YardPerMinute:
            return """yd/min"""
        
        if unit_abbreviation == SpeedUnits.YardPerHour:
            return """yd/h"""
        
        if unit_abbreviation == SpeedUnits.Knot:
            return """kn"""
        
        if unit_abbreviation == SpeedUnits.MilePerHour:
            return """mph"""
        
        if unit_abbreviation == SpeedUnits.Mach:
            return """M"""
        
        if unit_abbreviation == SpeedUnits.NanometerPerSecond:
            return """"""
        
        if unit_abbreviation == SpeedUnits.MicrometerPerSecond:
            return """"""
        
        if unit_abbreviation == SpeedUnits.MillimeterPerSecond:
            return """"""
        
        if unit_abbreviation == SpeedUnits.CentimeterPerSecond:
            return """"""
        
        if unit_abbreviation == SpeedUnits.DecimeterPerSecond:
            return """"""
        
        if unit_abbreviation == SpeedUnits.KilometerPerSecond:
            return """"""
        
        if unit_abbreviation == SpeedUnits.NanometerPerMinute:
            return """"""
        
        if unit_abbreviation == SpeedUnits.MicrometerPerMinute:
            return """"""
        
        if unit_abbreviation == SpeedUnits.MillimeterPerMinute:
            return """"""
        
        if unit_abbreviation == SpeedUnits.CentimeterPerMinute:
            return """"""
        
        if unit_abbreviation == SpeedUnits.DecimeterPerMinute:
            return """"""
        
        if unit_abbreviation == SpeedUnits.KilometerPerMinute:
            return """"""
        
        if unit_abbreviation == SpeedUnits.MillimeterPerHour:
            return """"""
        
        if unit_abbreviation == SpeedUnits.CentimeterPerHour:
            return """"""
        
        if unit_abbreviation == SpeedUnits.KilometerPerHour:
            return """"""
        

    def __str__(self):
        return self.to_string()


    def __add__(self, other):
        if not isinstance(other, Speed):
            raise TypeError("unsupported operand type(s) for +: 'Speed' and '{}'".format(type(other).__name__))
        return Speed(self.__value + other.__value)


    def __mul__(self, other):
        if not isinstance(other, Speed):
            raise TypeError("unsupported operand type(s) for *: 'Speed' and '{}'".format(type(other).__name__))
        return Speed(self.__value * other.__value)


    def __sub__(self, other):
        if not isinstance(other, Speed):
            raise TypeError("unsupported operand type(s) for -: 'Speed' and '{}'".format(type(other).__name__))
        return Speed(self.__value - other.__value)


    def __truediv__(self, other):
        if not isinstance(other, Speed):
            raise TypeError("unsupported operand type(s) for /: 'Speed' and '{}'".format(type(other).__name__))
        return Speed(self.__value / other.__value)


    def __mod__(self, other):
        if not isinstance(other, Speed):
            raise TypeError("unsupported operand type(s) for %: 'Speed' and '{}'".format(type(other).__name__))
        return Speed(self.__value % other.__value)


    def __pow__(self, other):
        if not isinstance(other, Speed):
            raise TypeError("unsupported operand type(s) for **: 'Speed' and '{}'".format(type(other).__name__))
        return Speed(self.__value ** other.__value)


    def __eq__(self, other):
        if not isinstance(other, Speed):
            raise TypeError("unsupported operand type(s) for ==: 'Speed' and '{}'".format(type(other).__name__))
        return self.__value == other.__value


    def __lt__(self, other):
        if not isinstance(other, Speed):
            raise TypeError("unsupported operand type(s) for <: 'Speed' and '{}'".format(type(other).__name__))
        return self.__value < other.__value


    def __gt__(self, other):
        if not isinstance(other, Speed):
            raise TypeError("unsupported operand type(s) for >: 'Speed' and '{}'".format(type(other).__name__))
        return self.__value > other.__value


    def __le__(self, other):
        if not isinstance(other, Speed):
            raise TypeError("unsupported operand type(s) for <=: 'Speed' and '{}'".format(type(other).__name__))
        return self.__value <= other.__value


    def __ge__(self, other):
        if not isinstance(other, Speed):
            raise TypeError("unsupported operand type(s) for >=: 'Speed' and '{}'".format(type(other).__name__))
        return self.__value >= other.__value


from enum import Enum
import math
import string


class KinematicViscosityUnits(Enum):
        """
            KinematicViscosityUnits enumeration
        """
        
        SquareMeterPerSecond = 'square_meter_per_second'
        """
            
        """
        
        Stokes = 'stokes'
        """
            
        """
        
        SquareFootPerSecond = 'square_foot_per_second'
        """
            
        """
        
        Nanostokes = 'nanostokes'
        """
            
        """
        
        Microstokes = 'microstokes'
        """
            
        """
        
        Millistokes = 'millistokes'
        """
            
        """
        
        Centistokes = 'centistokes'
        """
            
        """
        
        Decistokes = 'decistokes'
        """
            
        """
        
        Kilostokes = 'kilostokes'
        """
            
        """
        

class KinematicViscosity:
    """
    The viscosity of a fluid is a measure of its resistance to gradual deformation by shear stress or tensile stress.

    Args:
        value (float): The value.
        from_unit (KinematicViscosityUnits): The KinematicViscosity unit to create from, The default unit is SquareMeterPerSecond
    """
    def __init__(self, value: float, from_unit: KinematicViscosityUnits = KinematicViscosityUnits.SquareMeterPerSecond):
        if math.isnan(value):
            raise ValueError('Invalid unit: value is NaN')
        self.__value = self.__convert_to_base(value, from_unit)
        
        self.__square_meters_per_second = None
        
        self.__stokes = None
        
        self.__square_feet_per_second = None
        
        self.__nanostokes = None
        
        self.__microstokes = None
        
        self.__millistokes = None
        
        self.__centistokes = None
        
        self.__decistokes = None
        
        self.__kilostokes = None
        

    def __convert_from_base(self, from_unit: KinematicViscosityUnits) -> float:
        value = self.__value
        
        if from_unit == KinematicViscosityUnits.SquareMeterPerSecond:
            return (value)
        
        if from_unit == KinematicViscosityUnits.Stokes:
            return (value * 1e4)
        
        if from_unit == KinematicViscosityUnits.SquareFootPerSecond:
            return (value * 10.7639)
        
        if from_unit == KinematicViscosityUnits.Nanostokes:
            return ((value * 1e4) / 1e-09)
        
        if from_unit == KinematicViscosityUnits.Microstokes:
            return ((value * 1e4) / 1e-06)
        
        if from_unit == KinematicViscosityUnits.Millistokes:
            return ((value * 1e4) / 0.001)
        
        if from_unit == KinematicViscosityUnits.Centistokes:
            return ((value * 1e4) / 0.01)
        
        if from_unit == KinematicViscosityUnits.Decistokes:
            return ((value * 1e4) / 0.1)
        
        if from_unit == KinematicViscosityUnits.Kilostokes:
            return ((value * 1e4) / 1000.0)
        
        return None


    def __convert_to_base(self, value: float, to_unit: KinematicViscosityUnits) -> float:
        
        if to_unit == KinematicViscosityUnits.SquareMeterPerSecond:
            return (value)
        
        if to_unit == KinematicViscosityUnits.Stokes:
            return (value / 1e4)
        
        if to_unit == KinematicViscosityUnits.SquareFootPerSecond:
            return (value / 10.7639)
        
        if to_unit == KinematicViscosityUnits.Nanostokes:
            return ((value / 1e4) * 1e-09)
        
        if to_unit == KinematicViscosityUnits.Microstokes:
            return ((value / 1e4) * 1e-06)
        
        if to_unit == KinematicViscosityUnits.Millistokes:
            return ((value / 1e4) * 0.001)
        
        if to_unit == KinematicViscosityUnits.Centistokes:
            return ((value / 1e4) * 0.01)
        
        if to_unit == KinematicViscosityUnits.Decistokes:
            return ((value / 1e4) * 0.1)
        
        if to_unit == KinematicViscosityUnits.Kilostokes:
            return ((value / 1e4) * 1000.0)
        
        return None


    @property
    def base_value(self) -> float:
        return self.__value

    
    @staticmethod
    def from_square_meters_per_second(square_meters_per_second: float):
        """
        Create a new instance of KinematicViscosity from a value in square_meters_per_second.

        

        :param meters: The KinematicViscosity value in square_meters_per_second.
        :type square_meters_per_second: float
        :return: A new instance of KinematicViscosity.
        :rtype: KinematicViscosity
        """
        return KinematicViscosity(square_meters_per_second, KinematicViscosityUnits.SquareMeterPerSecond)

    
    @staticmethod
    def from_stokes(stokes: float):
        """
        Create a new instance of KinematicViscosity from a value in stokes.

        

        :param meters: The KinematicViscosity value in stokes.
        :type stokes: float
        :return: A new instance of KinematicViscosity.
        :rtype: KinematicViscosity
        """
        return KinematicViscosity(stokes, KinematicViscosityUnits.Stokes)

    
    @staticmethod
    def from_square_feet_per_second(square_feet_per_second: float):
        """
        Create a new instance of KinematicViscosity from a value in square_feet_per_second.

        

        :param meters: The KinematicViscosity value in square_feet_per_second.
        :type square_feet_per_second: float
        :return: A new instance of KinematicViscosity.
        :rtype: KinematicViscosity
        """
        return KinematicViscosity(square_feet_per_second, KinematicViscosityUnits.SquareFootPerSecond)

    
    @staticmethod
    def from_nanostokes(nanostokes: float):
        """
        Create a new instance of KinematicViscosity from a value in nanostokes.

        

        :param meters: The KinematicViscosity value in nanostokes.
        :type nanostokes: float
        :return: A new instance of KinematicViscosity.
        :rtype: KinematicViscosity
        """
        return KinematicViscosity(nanostokes, KinematicViscosityUnits.Nanostokes)

    
    @staticmethod
    def from_microstokes(microstokes: float):
        """
        Create a new instance of KinematicViscosity from a value in microstokes.

        

        :param meters: The KinematicViscosity value in microstokes.
        :type microstokes: float
        :return: A new instance of KinematicViscosity.
        :rtype: KinematicViscosity
        """
        return KinematicViscosity(microstokes, KinematicViscosityUnits.Microstokes)

    
    @staticmethod
    def from_millistokes(millistokes: float):
        """
        Create a new instance of KinematicViscosity from a value in millistokes.

        

        :param meters: The KinematicViscosity value in millistokes.
        :type millistokes: float
        :return: A new instance of KinematicViscosity.
        :rtype: KinematicViscosity
        """
        return KinematicViscosity(millistokes, KinematicViscosityUnits.Millistokes)

    
    @staticmethod
    def from_centistokes(centistokes: float):
        """
        Create a new instance of KinematicViscosity from a value in centistokes.

        

        :param meters: The KinematicViscosity value in centistokes.
        :type centistokes: float
        :return: A new instance of KinematicViscosity.
        :rtype: KinematicViscosity
        """
        return KinematicViscosity(centistokes, KinematicViscosityUnits.Centistokes)

    
    @staticmethod
    def from_decistokes(decistokes: float):
        """
        Create a new instance of KinematicViscosity from a value in decistokes.

        

        :param meters: The KinematicViscosity value in decistokes.
        :type decistokes: float
        :return: A new instance of KinematicViscosity.
        :rtype: KinematicViscosity
        """
        return KinematicViscosity(decistokes, KinematicViscosityUnits.Decistokes)

    
    @staticmethod
    def from_kilostokes(kilostokes: float):
        """
        Create a new instance of KinematicViscosity from a value in kilostokes.

        

        :param meters: The KinematicViscosity value in kilostokes.
        :type kilostokes: float
        :return: A new instance of KinematicViscosity.
        :rtype: KinematicViscosity
        """
        return KinematicViscosity(kilostokes, KinematicViscosityUnits.Kilostokes)

    
    @property
    def square_meters_per_second(self) -> float:
        """
        
        """
        if self.__square_meters_per_second != None:
            return self.__square_meters_per_second
        self.__square_meters_per_second = self.__convert_from_base(KinematicViscosityUnits.SquareMeterPerSecond)
        return self.__square_meters_per_second

    
    @property
    def stokes(self) -> float:
        """
        
        """
        if self.__stokes != None:
            return self.__stokes
        self.__stokes = self.__convert_from_base(KinematicViscosityUnits.Stokes)
        return self.__stokes

    
    @property
    def square_feet_per_second(self) -> float:
        """
        
        """
        if self.__square_feet_per_second != None:
            return self.__square_feet_per_second
        self.__square_feet_per_second = self.__convert_from_base(KinematicViscosityUnits.SquareFootPerSecond)
        return self.__square_feet_per_second

    
    @property
    def nanostokes(self) -> float:
        """
        
        """
        if self.__nanostokes != None:
            return self.__nanostokes
        self.__nanostokes = self.__convert_from_base(KinematicViscosityUnits.Nanostokes)
        return self.__nanostokes

    
    @property
    def microstokes(self) -> float:
        """
        
        """
        if self.__microstokes != None:
            return self.__microstokes
        self.__microstokes = self.__convert_from_base(KinematicViscosityUnits.Microstokes)
        return self.__microstokes

    
    @property
    def millistokes(self) -> float:
        """
        
        """
        if self.__millistokes != None:
            return self.__millistokes
        self.__millistokes = self.__convert_from_base(KinematicViscosityUnits.Millistokes)
        return self.__millistokes

    
    @property
    def centistokes(self) -> float:
        """
        
        """
        if self.__centistokes != None:
            return self.__centistokes
        self.__centistokes = self.__convert_from_base(KinematicViscosityUnits.Centistokes)
        return self.__centistokes

    
    @property
    def decistokes(self) -> float:
        """
        
        """
        if self.__decistokes != None:
            return self.__decistokes
        self.__decistokes = self.__convert_from_base(KinematicViscosityUnits.Decistokes)
        return self.__decistokes

    
    @property
    def kilostokes(self) -> float:
        """
        
        """
        if self.__kilostokes != None:
            return self.__kilostokes
        self.__kilostokes = self.__convert_from_base(KinematicViscosityUnits.Kilostokes)
        return self.__kilostokes

    
    def to_string(self, unit: KinematicViscosityUnits = KinematicViscosityUnits.SquareMeterPerSecond) -> string:
        """
        Format the KinematicViscosity to string.
        Note! the default format for KinematicViscosity is SquareMeterPerSecond.
        To specify the unit format set the 'unit' parameter.
        """
        
        if unit == KinematicViscosityUnits.SquareMeterPerSecond:
            return f"""{self.square_meters_per_second} m²/s"""
        
        if unit == KinematicViscosityUnits.Stokes:
            return f"""{self.stokes} St"""
        
        if unit == KinematicViscosityUnits.SquareFootPerSecond:
            return f"""{self.square_feet_per_second} ft²/s"""
        
        if unit == KinematicViscosityUnits.Nanostokes:
            return f"""{self.nanostokes} """
        
        if unit == KinematicViscosityUnits.Microstokes:
            return f"""{self.microstokes} """
        
        if unit == KinematicViscosityUnits.Millistokes:
            return f"""{self.millistokes} """
        
        if unit == KinematicViscosityUnits.Centistokes:
            return f"""{self.centistokes} """
        
        if unit == KinematicViscosityUnits.Decistokes:
            return f"""{self.decistokes} """
        
        if unit == KinematicViscosityUnits.Kilostokes:
            return f"""{self.kilostokes} """
        
        return f'{self.__value}'


    def get_unit_abbreviation(self, unit_abbreviation: KinematicViscosityUnits = KinematicViscosityUnits.SquareMeterPerSecond) -> string:
        """
        Get KinematicViscosity unit abbreviation.
        Note! the default abbreviation for KinematicViscosity is SquareMeterPerSecond.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == KinematicViscosityUnits.SquareMeterPerSecond:
            return """m²/s"""
        
        if unit_abbreviation == KinematicViscosityUnits.Stokes:
            return """St"""
        
        if unit_abbreviation == KinematicViscosityUnits.SquareFootPerSecond:
            return """ft²/s"""
        
        if unit_abbreviation == KinematicViscosityUnits.Nanostokes:
            return """"""
        
        if unit_abbreviation == KinematicViscosityUnits.Microstokes:
            return """"""
        
        if unit_abbreviation == KinematicViscosityUnits.Millistokes:
            return """"""
        
        if unit_abbreviation == KinematicViscosityUnits.Centistokes:
            return """"""
        
        if unit_abbreviation == KinematicViscosityUnits.Decistokes:
            return """"""
        
        if unit_abbreviation == KinematicViscosityUnits.Kilostokes:
            return """"""
        

    def __str__(self):
        return self.to_string()


    def __add__(self, other):
        if not isinstance(other, KinematicViscosity):
            raise TypeError("unsupported operand type(s) for +: 'KinematicViscosity' and '{}'".format(type(other).__name__))
        return KinematicViscosity(self.__value + other.__value)


    def __mul__(self, other):
        if not isinstance(other, KinematicViscosity):
            raise TypeError("unsupported operand type(s) for *: 'KinematicViscosity' and '{}'".format(type(other).__name__))
        return KinematicViscosity(self.__value * other.__value)


    def __sub__(self, other):
        if not isinstance(other, KinematicViscosity):
            raise TypeError("unsupported operand type(s) for -: 'KinematicViscosity' and '{}'".format(type(other).__name__))
        return KinematicViscosity(self.__value - other.__value)


    def __truediv__(self, other):
        if not isinstance(other, KinematicViscosity):
            raise TypeError("unsupported operand type(s) for /: 'KinematicViscosity' and '{}'".format(type(other).__name__))
        return KinematicViscosity(self.__value / other.__value)


    def __mod__(self, other):
        if not isinstance(other, KinematicViscosity):
            raise TypeError("unsupported operand type(s) for %: 'KinematicViscosity' and '{}'".format(type(other).__name__))
        return KinematicViscosity(self.__value % other.__value)


    def __pow__(self, other):
        if not isinstance(other, KinematicViscosity):
            raise TypeError("unsupported operand type(s) for **: 'KinematicViscosity' and '{}'".format(type(other).__name__))
        return KinematicViscosity(self.__value ** other.__value)


    def __eq__(self, other):
        if not isinstance(other, KinematicViscosity):
            raise TypeError("unsupported operand type(s) for ==: 'KinematicViscosity' and '{}'".format(type(other).__name__))
        return self.__value == other.__value


    def __lt__(self, other):
        if not isinstance(other, KinematicViscosity):
            raise TypeError("unsupported operand type(s) for <: 'KinematicViscosity' and '{}'".format(type(other).__name__))
        return self.__value < other.__value


    def __gt__(self, other):
        if not isinstance(other, KinematicViscosity):
            raise TypeError("unsupported operand type(s) for >: 'KinematicViscosity' and '{}'".format(type(other).__name__))
        return self.__value > other.__value


    def __le__(self, other):
        if not isinstance(other, KinematicViscosity):
            raise TypeError("unsupported operand type(s) for <=: 'KinematicViscosity' and '{}'".format(type(other).__name__))
        return self.__value <= other.__value


    def __ge__(self, other):
        if not isinstance(other, KinematicViscosity):
            raise TypeError("unsupported operand type(s) for >=: 'KinematicViscosity' and '{}'".format(type(other).__name__))
        return self.__value >= other.__value


from enum import Enum
import math
import string


class ApparentEnergyUnits(Enum):
        """
            ApparentEnergyUnits enumeration
        """
        
        VoltampereHour = 'voltampere_hour'
        """
            
        """
        
        KilovoltampereHour = 'kilovoltampere_hour'
        """
            
        """
        
        MegavoltampereHour = 'megavoltampere_hour'
        """
            
        """
        

class ApparentEnergy:
    """
    A unit for expressing the integral of apparent power over time, equal to the product of 1 volt-ampere and 1 hour, or to 3600 joules.

    Args:
        value (float): The value.
        from_unit (ApparentEnergyUnits): The ApparentEnergy unit to create from, The default unit is VoltampereHour
    """
    def __init__(self, value: float, from_unit: ApparentEnergyUnits = ApparentEnergyUnits.VoltampereHour):
        if math.isnan(value):
            raise ValueError('Invalid unit: value is NaN')
        self.__value = self.__convert_to_base(value, from_unit)
        
        self.__voltampere_hours = None
        
        self.__kilovoltampere_hours = None
        
        self.__megavoltampere_hours = None
        

    def __convert_from_base(self, from_unit: ApparentEnergyUnits) -> float:
        value = self.__value
        
        if from_unit == ApparentEnergyUnits.VoltampereHour:
            return (value)
        
        if from_unit == ApparentEnergyUnits.KilovoltampereHour:
            return ((value) / 1000.0)
        
        if from_unit == ApparentEnergyUnits.MegavoltampereHour:
            return ((value) / 1000000.0)
        
        return None


    def __convert_to_base(self, value: float, to_unit: ApparentEnergyUnits) -> float:
        
        if to_unit == ApparentEnergyUnits.VoltampereHour:
            return (value)
        
        if to_unit == ApparentEnergyUnits.KilovoltampereHour:
            return ((value) * 1000.0)
        
        if to_unit == ApparentEnergyUnits.MegavoltampereHour:
            return ((value) * 1000000.0)
        
        return None


    @property
    def base_value(self) -> float:
        return self.__value

    
    @staticmethod
    def from_voltampere_hours(voltampere_hours: float):
        """
        Create a new instance of ApparentEnergy from a value in voltampere_hours.

        

        :param meters: The ApparentEnergy value in voltampere_hours.
        :type voltampere_hours: float
        :return: A new instance of ApparentEnergy.
        :rtype: ApparentEnergy
        """
        return ApparentEnergy(voltampere_hours, ApparentEnergyUnits.VoltampereHour)

    
    @staticmethod
    def from_kilovoltampere_hours(kilovoltampere_hours: float):
        """
        Create a new instance of ApparentEnergy from a value in kilovoltampere_hours.

        

        :param meters: The ApparentEnergy value in kilovoltampere_hours.
        :type kilovoltampere_hours: float
        :return: A new instance of ApparentEnergy.
        :rtype: ApparentEnergy
        """
        return ApparentEnergy(kilovoltampere_hours, ApparentEnergyUnits.KilovoltampereHour)

    
    @staticmethod
    def from_megavoltampere_hours(megavoltampere_hours: float):
        """
        Create a new instance of ApparentEnergy from a value in megavoltampere_hours.

        

        :param meters: The ApparentEnergy value in megavoltampere_hours.
        :type megavoltampere_hours: float
        :return: A new instance of ApparentEnergy.
        :rtype: ApparentEnergy
        """
        return ApparentEnergy(megavoltampere_hours, ApparentEnergyUnits.MegavoltampereHour)

    
    @property
    def voltampere_hours(self) -> float:
        """
        
        """
        if self.__voltampere_hours != None:
            return self.__voltampere_hours
        self.__voltampere_hours = self.__convert_from_base(ApparentEnergyUnits.VoltampereHour)
        return self.__voltampere_hours

    
    @property
    def kilovoltampere_hours(self) -> float:
        """
        
        """
        if self.__kilovoltampere_hours != None:
            return self.__kilovoltampere_hours
        self.__kilovoltampere_hours = self.__convert_from_base(ApparentEnergyUnits.KilovoltampereHour)
        return self.__kilovoltampere_hours

    
    @property
    def megavoltampere_hours(self) -> float:
        """
        
        """
        if self.__megavoltampere_hours != None:
            return self.__megavoltampere_hours
        self.__megavoltampere_hours = self.__convert_from_base(ApparentEnergyUnits.MegavoltampereHour)
        return self.__megavoltampere_hours

    
    def to_string(self, unit: ApparentEnergyUnits = ApparentEnergyUnits.VoltampereHour) -> string:
        """
        Format the ApparentEnergy to string.
        Note! the default format for ApparentEnergy is VoltampereHour.
        To specify the unit format set the 'unit' parameter.
        """
        
        if unit == ApparentEnergyUnits.VoltampereHour:
            return f"""{self.voltampere_hours} VAh"""
        
        if unit == ApparentEnergyUnits.KilovoltampereHour:
            return f"""{self.kilovoltampere_hours} """
        
        if unit == ApparentEnergyUnits.MegavoltampereHour:
            return f"""{self.megavoltampere_hours} """
        
        return f'{self.__value}'


    def get_unit_abbreviation(self, unit_abbreviation: ApparentEnergyUnits = ApparentEnergyUnits.VoltampereHour) -> string:
        """
        Get ApparentEnergy unit abbreviation.
        Note! the default abbreviation for ApparentEnergy is VoltampereHour.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == ApparentEnergyUnits.VoltampereHour:
            return """VAh"""
        
        if unit_abbreviation == ApparentEnergyUnits.KilovoltampereHour:
            return """"""
        
        if unit_abbreviation == ApparentEnergyUnits.MegavoltampereHour:
            return """"""
        

    def __str__(self):
        return self.to_string()


    def __add__(self, other):
        if not isinstance(other, ApparentEnergy):
            raise TypeError("unsupported operand type(s) for +: 'ApparentEnergy' and '{}'".format(type(other).__name__))
        return ApparentEnergy(self.__value + other.__value)


    def __mul__(self, other):
        if not isinstance(other, ApparentEnergy):
            raise TypeError("unsupported operand type(s) for *: 'ApparentEnergy' and '{}'".format(type(other).__name__))
        return ApparentEnergy(self.__value * other.__value)


    def __sub__(self, other):
        if not isinstance(other, ApparentEnergy):
            raise TypeError("unsupported operand type(s) for -: 'ApparentEnergy' and '{}'".format(type(other).__name__))
        return ApparentEnergy(self.__value - other.__value)


    def __truediv__(self, other):
        if not isinstance(other, ApparentEnergy):
            raise TypeError("unsupported operand type(s) for /: 'ApparentEnergy' and '{}'".format(type(other).__name__))
        return ApparentEnergy(self.__value / other.__value)


    def __mod__(self, other):
        if not isinstance(other, ApparentEnergy):
            raise TypeError("unsupported operand type(s) for %: 'ApparentEnergy' and '{}'".format(type(other).__name__))
        return ApparentEnergy(self.__value % other.__value)


    def __pow__(self, other):
        if not isinstance(other, ApparentEnergy):
            raise TypeError("unsupported operand type(s) for **: 'ApparentEnergy' and '{}'".format(type(other).__name__))
        return ApparentEnergy(self.__value ** other.__value)


    def __eq__(self, other):
        if not isinstance(other, ApparentEnergy):
            raise TypeError("unsupported operand type(s) for ==: 'ApparentEnergy' and '{}'".format(type(other).__name__))
        return self.__value == other.__value


    def __lt__(self, other):
        if not isinstance(other, ApparentEnergy):
            raise TypeError("unsupported operand type(s) for <: 'ApparentEnergy' and '{}'".format(type(other).__name__))
        return self.__value < other.__value


    def __gt__(self, other):
        if not isinstance(other, ApparentEnergy):
            raise TypeError("unsupported operand type(s) for >: 'ApparentEnergy' and '{}'".format(type(other).__name__))
        return self.__value > other.__value


    def __le__(self, other):
        if not isinstance(other, ApparentEnergy):
            raise TypeError("unsupported operand type(s) for <=: 'ApparentEnergy' and '{}'".format(type(other).__name__))
        return self.__value <= other.__value


    def __ge__(self, other):
        if not isinstance(other, ApparentEnergy):
            raise TypeError("unsupported operand type(s) for >=: 'ApparentEnergy' and '{}'".format(type(other).__name__))
        return self.__value >= other.__value